FLAG_ALPHA_FEATURES_NAME = "alpha_features"
FLAG_ON_DEMAND_TRANSFORM_NAME = "on_demand_transforms"
FLAG_PYTHON_FEATURE_SERVER_NAME = "python_feature_server"
FLAG_AWS_LAMBDA_FEATURE_SERVER_NAME = "aws_lambda_feature_server"
FLAG_DIRECT_INGEST_TO_ONLINE_STORE = "direct_ingest_to_online_store"
ENV_FLAG_IS_TEST = "IS_TEST"

FLAG_NAMES = {
    FLAG_ALPHA_FEATURES_NAME,
    FLAG_ON_DEMAND_TRANSFORM_NAME,
    FLAG_PYTHON_FEATURE_SERVER_NAME,
    FLAG_AWS_LAMBDA_FEATURE_SERVER_NAME,
    FLAG_DIRECT_INGEST_TO_ONLINE_STORE,
}
