# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/RequestFeatureView.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from feast.protos.feast.core import FeatureView_pb2 as feast_dot_core_dot_FeatureView__pb2
from feast.protos.feast.core import Feature_pb2 as feast_dot_core_dot_Feature__pb2
from feast.protos.feast.core import DataSource_pb2 as feast_dot_core_dot_DataSource__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='feast/core/RequestFeatureView.proto',
  package='feast.core',
  syntax='proto3',
  serialized_options=b'\n\020feast.proto.coreB\027RequestFeatureViewProtoZ3github.com/feast-dev/feast/sdk/go/protos/feast/core',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n#feast/core/RequestFeatureView.proto\x12\nfeast.core\x1a\x1c\x66\x65\x61st/core/FeatureView.proto\x1a\x18\x66\x65\x61st/core/Feature.proto\x1a\x1b\x66\x65\x61st/core/DataSource.proto\"F\n\x12RequestFeatureView\x12\x30\n\x04spec\x18\x01 \x01(\x0b\x32\".feast.core.RequestFeatureViewSpec\"l\n\x16RequestFeatureViewSpec\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x33\n\x13request_data_source\x18\x03 \x01(\x0b\x32\x16.feast.core.DataSourceB`\n\x10\x66\x65\x61st.proto.coreB\x17RequestFeatureViewProtoZ3github.com/feast-dev/feast/sdk/go/protos/feast/coreb\x06proto3'
  ,
  dependencies=[feast_dot_core_dot_FeatureView__pb2.DESCRIPTOR,feast_dot_core_dot_Feature__pb2.DESCRIPTOR,feast_dot_core_dot_DataSource__pb2.DESCRIPTOR,])




_REQUESTFEATUREVIEW = _descriptor.Descriptor(
  name='RequestFeatureView',
  full_name='feast.core.RequestFeatureView',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='spec', full_name='feast.core.RequestFeatureView.spec', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=136,
  serialized_end=206,
)


_REQUESTFEATUREVIEWSPEC = _descriptor.Descriptor(
  name='RequestFeatureViewSpec',
  full_name='feast.core.RequestFeatureViewSpec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='feast.core.RequestFeatureViewSpec.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='project', full_name='feast.core.RequestFeatureViewSpec.project', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='request_data_source', full_name='feast.core.RequestFeatureViewSpec.request_data_source', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=208,
  serialized_end=316,
)

_REQUESTFEATUREVIEW.fields_by_name['spec'].message_type = _REQUESTFEATUREVIEWSPEC
_REQUESTFEATUREVIEWSPEC.fields_by_name['request_data_source'].message_type = feast_dot_core_dot_DataSource__pb2._DATASOURCE
DESCRIPTOR.message_types_by_name['RequestFeatureView'] = _REQUESTFEATUREVIEW
DESCRIPTOR.message_types_by_name['RequestFeatureViewSpec'] = _REQUESTFEATUREVIEWSPEC
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

RequestFeatureView = _reflection.GeneratedProtocolMessageType('RequestFeatureView', (_message.Message,), {
  'DESCRIPTOR' : _REQUESTFEATUREVIEW,
  '__module__' : 'feast.core.RequestFeatureView_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.RequestFeatureView)
  })
_sym_db.RegisterMessage(RequestFeatureView)

RequestFeatureViewSpec = _reflection.GeneratedProtocolMessageType('RequestFeatureViewSpec', (_message.Message,), {
  'DESCRIPTOR' : _REQUESTFEATUREVIEWSPEC,
  '__module__' : 'feast.core.RequestFeatureView_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.RequestFeatureViewSpec)
  })
_sym_db.RegisterMessage(RequestFeatureViewSpec)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
