# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/serving/TransformationService.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='feast/serving/TransformationService.proto',
  package='feast.serving',
  syntax='proto3',
  serialized_options=b'\n\023feast.proto.servingB\035TransformationServiceAPIProtoZ6github.com/feast-dev/feast/sdk/go/protos/feast/serving',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n)feast/serving/TransformationService.proto\x12\rfeast.serving\"+\n\tValueType\x12\x15\n\x0b\x61rrow_value\x18\x01 \x01(\x0cH\x00\x42\x07\n\x05value\"%\n#GetTransformationServiceInfoRequest\"\x9c\x01\n$GetTransformationServiceInfoResponse\x12\x0f\n\x07version\x18\x01 \x01(\t\x12\x36\n\x04type\x18\x02 \x01(\x0e\x32(.feast.serving.TransformationServiceType\x12+\n#transformation_service_type_details\x18\x03 \x01(\t\"\x88\x01\n\x18TransformFeaturesRequest\x12#\n\x1bon_demand_feature_view_name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x36\n\x14transformation_input\x18\x03 \x01(\x0b\x32\x18.feast.serving.ValueType\"T\n\x19TransformFeaturesResponse\x12\x37\n\x15transformation_output\x18\x03 \x01(\x0b\x32\x18.feast.serving.ValueType*\x94\x01\n\x19TransformationServiceType\x12\'\n#TRANSFORMATION_SERVICE_TYPE_INVALID\x10\x00\x12&\n\"TRANSFORMATION_SERVICE_TYPE_PYTHON\x10\x01\x12&\n\"TRANSFORMATION_SERVICE_TYPE_CUSTOM\x10\x64\x32\x89\x02\n\x15TransformationService\x12\x87\x01\n\x1cGetTransformationServiceInfo\x12\x32.feast.serving.GetTransformationServiceInfoRequest\x1a\x33.feast.serving.GetTransformationServiceInfoResponse\x12\x66\n\x11TransformFeatures\x12\'.feast.serving.TransformFeaturesRequest\x1a(.feast.serving.TransformFeaturesResponseBl\n\x13\x66\x65\x61st.proto.servingB\x1dTransformationServiceAPIProtoZ6github.com/feast-dev/feast/sdk/go/protos/feast/servingb\x06proto3'
)

_TRANSFORMATIONSERVICETYPE = _descriptor.EnumDescriptor(
  name='TransformationServiceType',
  full_name='feast.serving.TransformationServiceType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='TRANSFORMATION_SERVICE_TYPE_INVALID', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TRANSFORMATION_SERVICE_TYPE_PYTHON', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TRANSFORMATION_SERVICE_TYPE_CUSTOM', index=2, number=100,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=529,
  serialized_end=677,
)
_sym_db.RegisterEnumDescriptor(_TRANSFORMATIONSERVICETYPE)

TransformationServiceType = enum_type_wrapper.EnumTypeWrapper(_TRANSFORMATIONSERVICETYPE)
TRANSFORMATION_SERVICE_TYPE_INVALID = 0
TRANSFORMATION_SERVICE_TYPE_PYTHON = 1
TRANSFORMATION_SERVICE_TYPE_CUSTOM = 100



_VALUETYPE = _descriptor.Descriptor(
  name='ValueType',
  full_name='feast.serving.ValueType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='arrow_value', full_name='feast.serving.ValueType.arrow_value', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='value', full_name='feast.serving.ValueType.value',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=60,
  serialized_end=103,
)


_GETTRANSFORMATIONSERVICEINFOREQUEST = _descriptor.Descriptor(
  name='GetTransformationServiceInfoRequest',
  full_name='feast.serving.GetTransformationServiceInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=105,
  serialized_end=142,
)


_GETTRANSFORMATIONSERVICEINFORESPONSE = _descriptor.Descriptor(
  name='GetTransformationServiceInfoResponse',
  full_name='feast.serving.GetTransformationServiceInfoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='feast.serving.GetTransformationServiceInfoResponse.version', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='feast.serving.GetTransformationServiceInfoResponse.type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='transformation_service_type_details', full_name='feast.serving.GetTransformationServiceInfoResponse.transformation_service_type_details', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=145,
  serialized_end=301,
)


_TRANSFORMFEATURESREQUEST = _descriptor.Descriptor(
  name='TransformFeaturesRequest',
  full_name='feast.serving.TransformFeaturesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='on_demand_feature_view_name', full_name='feast.serving.TransformFeaturesRequest.on_demand_feature_view_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='project', full_name='feast.serving.TransformFeaturesRequest.project', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='transformation_input', full_name='feast.serving.TransformFeaturesRequest.transformation_input', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=304,
  serialized_end=440,
)


_TRANSFORMFEATURESRESPONSE = _descriptor.Descriptor(
  name='TransformFeaturesResponse',
  full_name='feast.serving.TransformFeaturesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='transformation_output', full_name='feast.serving.TransformFeaturesResponse.transformation_output', index=0,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=442,
  serialized_end=526,
)

_VALUETYPE.oneofs_by_name['value'].fields.append(
  _VALUETYPE.fields_by_name['arrow_value'])
_VALUETYPE.fields_by_name['arrow_value'].containing_oneof = _VALUETYPE.oneofs_by_name['value']
_GETTRANSFORMATIONSERVICEINFORESPONSE.fields_by_name['type'].enum_type = _TRANSFORMATIONSERVICETYPE
_TRANSFORMFEATURESREQUEST.fields_by_name['transformation_input'].message_type = _VALUETYPE
_TRANSFORMFEATURESRESPONSE.fields_by_name['transformation_output'].message_type = _VALUETYPE
DESCRIPTOR.message_types_by_name['ValueType'] = _VALUETYPE
DESCRIPTOR.message_types_by_name['GetTransformationServiceInfoRequest'] = _GETTRANSFORMATIONSERVICEINFOREQUEST
DESCRIPTOR.message_types_by_name['GetTransformationServiceInfoResponse'] = _GETTRANSFORMATIONSERVICEINFORESPONSE
DESCRIPTOR.message_types_by_name['TransformFeaturesRequest'] = _TRANSFORMFEATURESREQUEST
DESCRIPTOR.message_types_by_name['TransformFeaturesResponse'] = _TRANSFORMFEATURESRESPONSE
DESCRIPTOR.enum_types_by_name['TransformationServiceType'] = _TRANSFORMATIONSERVICETYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ValueType = _reflection.GeneratedProtocolMessageType('ValueType', (_message.Message,), {
  'DESCRIPTOR' : _VALUETYPE,
  '__module__' : 'feast.serving.TransformationService_pb2'
  # @@protoc_insertion_point(class_scope:feast.serving.ValueType)
  })
_sym_db.RegisterMessage(ValueType)

GetTransformationServiceInfoRequest = _reflection.GeneratedProtocolMessageType('GetTransformationServiceInfoRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETTRANSFORMATIONSERVICEINFOREQUEST,
  '__module__' : 'feast.serving.TransformationService_pb2'
  # @@protoc_insertion_point(class_scope:feast.serving.GetTransformationServiceInfoRequest)
  })
_sym_db.RegisterMessage(GetTransformationServiceInfoRequest)

GetTransformationServiceInfoResponse = _reflection.GeneratedProtocolMessageType('GetTransformationServiceInfoResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETTRANSFORMATIONSERVICEINFORESPONSE,
  '__module__' : 'feast.serving.TransformationService_pb2'
  # @@protoc_insertion_point(class_scope:feast.serving.GetTransformationServiceInfoResponse)
  })
_sym_db.RegisterMessage(GetTransformationServiceInfoResponse)

TransformFeaturesRequest = _reflection.GeneratedProtocolMessageType('TransformFeaturesRequest', (_message.Message,), {
  'DESCRIPTOR' : _TRANSFORMFEATURESREQUEST,
  '__module__' : 'feast.serving.TransformationService_pb2'
  # @@protoc_insertion_point(class_scope:feast.serving.TransformFeaturesRequest)
  })
_sym_db.RegisterMessage(TransformFeaturesRequest)

TransformFeaturesResponse = _reflection.GeneratedProtocolMessageType('TransformFeaturesResponse', (_message.Message,), {
  'DESCRIPTOR' : _TRANSFORMFEATURESRESPONSE,
  '__module__' : 'feast.serving.TransformationService_pb2'
  # @@protoc_insertion_point(class_scope:feast.serving.TransformFeaturesResponse)
  })
_sym_db.RegisterMessage(TransformFeaturesResponse)


DESCRIPTOR._options = None

_TRANSFORMATIONSERVICE = _descriptor.ServiceDescriptor(
  name='TransformationService',
  full_name='feast.serving.TransformationService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=680,
  serialized_end=945,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetTransformationServiceInfo',
    full_name='feast.serving.TransformationService.GetTransformationServiceInfo',
    index=0,
    containing_service=None,
    input_type=_GETTRANSFORMATIONSERVICEINFOREQUEST,
    output_type=_GETTRANSFORMATIONSERVICEINFORESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='TransformFeatures',
    full_name='feast.serving.TransformationService.TransformFeatures',
    index=1,
    containing_service=None,
    input_type=_TRANSFORMFEATURESREQUEST,
    output_type=_TRANSFORMFEATURESRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_TRANSFORMATIONSERVICE)

DESCRIPTOR.services_by_name['TransformationService'] = _TRANSFORMATIONSERVICE

# @@protoc_insertion_point(module_scope)
