# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/FeatureService.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from feast.protos.feast.core import FeatureViewProjection_pb2 as feast_dot_core_dot_FeatureViewProjection__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='feast/core/FeatureService.proto',
  package='feast.core',
  syntax='proto3',
  serialized_options=b'\n\020feast.proto.coreB\023FeatureServiceProtoZ3github.com/feast-dev/feast/sdk/go/protos/feast/core',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1f\x66\x65\x61st/core/FeatureService.proto\x12\nfeast.core\x1a\x1fgoogle/protobuf/timestamp.proto\x1a&feast/core/FeatureViewProjection.proto\"l\n\x0e\x46\x65\x61tureService\x12,\n\x04spec\x18\x01 \x01(\x0b\x32\x1e.feast.core.FeatureServiceSpec\x12,\n\x04meta\x18\x02 \x01(\x0b\x32\x1e.feast.core.FeatureServiceMeta\"\xe2\x01\n\x12\x46\x65\x61tureServiceSpec\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x33\n\x08\x66\x65\x61tures\x18\x03 \x03(\x0b\x32!.feast.core.FeatureViewProjection\x12\x36\n\x04tags\x18\x04 \x03(\x0b\x32(.feast.core.FeatureServiceSpec.TagsEntry\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x87\x01\n\x12\x46\x65\x61tureServiceMeta\x12\x35\n\x11\x63reated_timestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12:\n\x16last_updated_timestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\\\n\x10\x66\x65\x61st.proto.coreB\x13\x46\x65\x61tureServiceProtoZ3github.com/feast-dev/feast/sdk/go/protos/feast/coreb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,feast_dot_core_dot_FeatureViewProjection__pb2.DESCRIPTOR,])




_FEATURESERVICE = _descriptor.Descriptor(
  name='FeatureService',
  full_name='feast.core.FeatureService',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='spec', full_name='feast.core.FeatureService.spec', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='meta', full_name='feast.core.FeatureService.meta', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=120,
  serialized_end=228,
)


_FEATURESERVICESPEC_TAGSENTRY = _descriptor.Descriptor(
  name='TagsEntry',
  full_name='feast.core.FeatureServiceSpec.TagsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='feast.core.FeatureServiceSpec.TagsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='feast.core.FeatureServiceSpec.TagsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=414,
  serialized_end=457,
)

_FEATURESERVICESPEC = _descriptor.Descriptor(
  name='FeatureServiceSpec',
  full_name='feast.core.FeatureServiceSpec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='feast.core.FeatureServiceSpec.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='project', full_name='feast.core.FeatureServiceSpec.project', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='features', full_name='feast.core.FeatureServiceSpec.features', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tags', full_name='feast.core.FeatureServiceSpec.tags', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='description', full_name='feast.core.FeatureServiceSpec.description', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_FEATURESERVICESPEC_TAGSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=231,
  serialized_end=457,
)


_FEATURESERVICEMETA = _descriptor.Descriptor(
  name='FeatureServiceMeta',
  full_name='feast.core.FeatureServiceMeta',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='created_timestamp', full_name='feast.core.FeatureServiceMeta.created_timestamp', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='last_updated_timestamp', full_name='feast.core.FeatureServiceMeta.last_updated_timestamp', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=460,
  serialized_end=595,
)

_FEATURESERVICE.fields_by_name['spec'].message_type = _FEATURESERVICESPEC
_FEATURESERVICE.fields_by_name['meta'].message_type = _FEATURESERVICEMETA
_FEATURESERVICESPEC_TAGSENTRY.containing_type = _FEATURESERVICESPEC
_FEATURESERVICESPEC.fields_by_name['features'].message_type = feast_dot_core_dot_FeatureViewProjection__pb2._FEATUREVIEWPROJECTION
_FEATURESERVICESPEC.fields_by_name['tags'].message_type = _FEATURESERVICESPEC_TAGSENTRY
_FEATURESERVICEMETA.fields_by_name['created_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_FEATURESERVICEMETA.fields_by_name['last_updated_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
DESCRIPTOR.message_types_by_name['FeatureService'] = _FEATURESERVICE
DESCRIPTOR.message_types_by_name['FeatureServiceSpec'] = _FEATURESERVICESPEC
DESCRIPTOR.message_types_by_name['FeatureServiceMeta'] = _FEATURESERVICEMETA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FeatureService = _reflection.GeneratedProtocolMessageType('FeatureService', (_message.Message,), {
  'DESCRIPTOR' : _FEATURESERVICE,
  '__module__' : 'feast.core.FeatureService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.FeatureService)
  })
_sym_db.RegisterMessage(FeatureService)

FeatureServiceSpec = _reflection.GeneratedProtocolMessageType('FeatureServiceSpec', (_message.Message,), {

  'TagsEntry' : _reflection.GeneratedProtocolMessageType('TagsEntry', (_message.Message,), {
    'DESCRIPTOR' : _FEATURESERVICESPEC_TAGSENTRY,
    '__module__' : 'feast.core.FeatureService_pb2'
    # @@protoc_insertion_point(class_scope:feast.core.FeatureServiceSpec.TagsEntry)
    })
  ,
  'DESCRIPTOR' : _FEATURESERVICESPEC,
  '__module__' : 'feast.core.FeatureService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.FeatureServiceSpec)
  })
_sym_db.RegisterMessage(FeatureServiceSpec)
_sym_db.RegisterMessage(FeatureServiceSpec.TagsEntry)

FeatureServiceMeta = _reflection.GeneratedProtocolMessageType('FeatureServiceMeta', (_message.Message,), {
  'DESCRIPTOR' : _FEATURESERVICEMETA,
  '__module__' : 'feast.core.FeatureService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.FeatureServiceMeta)
  })
_sym_db.RegisterMessage(FeatureServiceMeta)


DESCRIPTOR._options = None
_FEATURESERVICESPEC_TAGSENTRY._options = None
# @@protoc_insertion_point(module_scope)
