# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/CoreService.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tensorflow_metadata.proto.v0 import statistics_pb2 as tensorflow__metadata_dot_proto_dot_v0_dot_statistics__pb2
from feast.protos.feast.core import Entity_pb2 as feast_dot_core_dot_Entity__pb2
from feast.protos.feast.core import Feature_pb2 as feast_dot_core_dot_Feature__pb2
from feast.protos.feast.core import FeatureTable_pb2 as feast_dot_core_dot_FeatureTable__pb2
from feast.protos.feast.core import Store_pb2 as feast_dot_core_dot_Store__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='feast/core/CoreService.proto',
  package='feast.core',
  syntax='proto3',
  serialized_options=b'\n\020feast.proto.coreB\020CoreServiceProtoZ3github.com/feast-dev/feast/sdk/go/protos/feast/core',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1c\x66\x65\x61st/core/CoreService.proto\x12\nfeast.core\x1a\x1fgoogle/protobuf/timestamp.proto\x1a-tensorflow_metadata/proto/v0/statistics.proto\x1a\x17\x66\x65\x61st/core/Entity.proto\x1a\x18\x66\x65\x61st/core/Feature.proto\x1a\x1d\x66\x65\x61st/core/FeatureTable.proto\x1a\x16\x66\x65\x61st/core/Store.proto\"1\n\x10GetEntityRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\"7\n\x11GetEntityResponse\x12\"\n\x06\x65ntity\x18\x01 \x01(\x0b\x32\x12.feast.core.Entity\"\xdc\x01\n\x13ListEntitiesRequest\x12\x36\n\x06\x66ilter\x18\x01 \x01(\x0b\x32&.feast.core.ListEntitiesRequest.Filter\x1a\x8c\x01\n\x06\x46ilter\x12\x0f\n\x07project\x18\x03 \x01(\t\x12\x42\n\x06labels\x18\x04 \x03(\x0b\x32\x32.feast.core.ListEntitiesRequest.Filter.LabelsEntry\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"<\n\x14ListEntitiesResponse\x12$\n\x08\x65ntities\x18\x01 \x03(\x0b\x32\x12.feast.core.Entity\"\xee\x01\n\x13ListFeaturesRequest\x12\x36\n\x06\x66ilter\x18\x01 \x01(\x0b\x32&.feast.core.ListFeaturesRequest.Filter\x1a\x9e\x01\n\x06\x46ilter\x12\x42\n\x06labels\x18\x01 \x03(\x0b\x32\x32.feast.core.ListFeaturesRequest.Filter.LabelsEntry\x12\x10\n\x08\x65ntities\x18\x02 \x03(\t\x12\x0f\n\x07project\x18\x03 \x01(\t\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xaa\x01\n\x14ListFeaturesResponse\x12@\n\x08\x66\x65\x61tures\x18\x02 \x03(\x0b\x32..feast.core.ListFeaturesResponse.FeaturesEntry\x1aJ\n\rFeaturesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.feast.core.FeatureSpecV2:\x02\x38\x01J\x04\x08\x01\x10\x02\"a\n\x11ListStoresRequest\x12\x34\n\x06\x66ilter\x18\x01 \x01(\x0b\x32$.feast.core.ListStoresRequest.Filter\x1a\x16\n\x06\x46ilter\x12\x0c\n\x04name\x18\x01 \x01(\t\"6\n\x12ListStoresResponse\x12 \n\x05store\x18\x01 \x03(\x0b\x32\x11.feast.core.Store\"M\n\x12\x41pplyEntityRequest\x12&\n\x04spec\x18\x01 \x01(\x0b\x32\x18.feast.core.EntitySpecV2\x12\x0f\n\x07project\x18\x02 \x01(\t\"9\n\x13\x41pplyEntityResponse\x12\"\n\x06\x65ntity\x18\x01 \x01(\x0b\x32\x12.feast.core.Entity\"\x1c\n\x1aGetFeastCoreVersionRequest\".\n\x1bGetFeastCoreVersionResponse\x12\x0f\n\x07version\x18\x01 \x01(\t\"6\n\x12UpdateStoreRequest\x12 \n\x05store\x18\x01 \x01(\x0b\x32\x11.feast.core.Store\"\x95\x01\n\x13UpdateStoreResponse\x12 \n\x05store\x18\x01 \x01(\x0b\x32\x11.feast.core.Store\x12\x36\n\x06status\x18\x02 \x01(\x0e\x32&.feast.core.UpdateStoreResponse.Status\"$\n\x06Status\x12\r\n\tNO_CHANGE\x10\x00\x12\x0b\n\x07UPDATED\x10\x01\"$\n\x14\x43reateProjectRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"\x17\n\x15\x43reateProjectResponse\"%\n\x15\x41rchiveProjectRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"\x18\n\x16\x41rchiveProjectResponse\"\x15\n\x13ListProjectsRequest\"(\n\x14ListProjectsResponse\x12\x10\n\x08projects\x18\x01 \x03(\t\" \n\x1eUpdateFeatureSetStatusResponse\"]\n\x18\x41pplyFeatureTableRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x30\n\ntable_spec\x18\x02 \x01(\x0b\x32\x1c.feast.core.FeatureTableSpec\"D\n\x19\x41pplyFeatureTableResponse\x12\'\n\x05table\x18\x01 \x01(\x0b\x32\x18.feast.core.FeatureTable\"7\n\x16GetFeatureTableRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\"B\n\x17GetFeatureTableResponse\x12\'\n\x05table\x18\x01 \x01(\x0b\x32\x18.feast.core.FeatureTable\"\xeb\x01\n\x18ListFeatureTablesRequest\x12;\n\x06\x66ilter\x18\x01 \x01(\x0b\x32+.feast.core.ListFeatureTablesRequest.Filter\x1a\x91\x01\n\x06\x46ilter\x12\x0f\n\x07project\x18\x01 \x01(\t\x12G\n\x06labels\x18\x03 \x03(\x0b\x32\x37.feast.core.ListFeatureTablesRequest.Filter.LabelsEntry\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"E\n\x19ListFeatureTablesResponse\x12(\n\x06tables\x18\x01 \x03(\x0b\x32\x18.feast.core.FeatureTable\":\n\x19\x44\x65leteFeatureTableRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\"\x1c\n\x1a\x44\x65leteFeatureTableResponse2\xd9\t\n\x0b\x43oreService\x12\x66\n\x13GetFeastCoreVersion\x12&.feast.core.GetFeastCoreVersionRequest\x1a\'.feast.core.GetFeastCoreVersionResponse\x12H\n\tGetEntity\x12\x1c.feast.core.GetEntityRequest\x1a\x1d.feast.core.GetEntityResponse\x12Q\n\x0cListFeatures\x12\x1f.feast.core.ListFeaturesRequest\x1a .feast.core.ListFeaturesResponse\x12K\n\nListStores\x12\x1d.feast.core.ListStoresRequest\x1a\x1e.feast.core.ListStoresResponse\x12N\n\x0b\x41pplyEntity\x12\x1e.feast.core.ApplyEntityRequest\x1a\x1f.feast.core.ApplyEntityResponse\x12Q\n\x0cListEntities\x12\x1f.feast.core.ListEntitiesRequest\x1a .feast.core.ListEntitiesResponse\x12N\n\x0bUpdateStore\x12\x1e.feast.core.UpdateStoreRequest\x1a\x1f.feast.core.UpdateStoreResponse\x12T\n\rCreateProject\x12 .feast.core.CreateProjectRequest\x1a!.feast.core.CreateProjectResponse\x12W\n\x0e\x41rchiveProject\x12!.feast.core.ArchiveProjectRequest\x1a\".feast.core.ArchiveProjectResponse\x12Q\n\x0cListProjects\x12\x1f.feast.core.ListProjectsRequest\x1a .feast.core.ListProjectsResponse\x12`\n\x11\x41pplyFeatureTable\x12$.feast.core.ApplyFeatureTableRequest\x1a%.feast.core.ApplyFeatureTableResponse\x12`\n\x11ListFeatureTables\x12$.feast.core.ListFeatureTablesRequest\x1a%.feast.core.ListFeatureTablesResponse\x12Z\n\x0fGetFeatureTable\x12\".feast.core.GetFeatureTableRequest\x1a#.feast.core.GetFeatureTableResponse\x12\x63\n\x12\x44\x65leteFeatureTable\x12%.feast.core.DeleteFeatureTableRequest\x1a&.feast.core.DeleteFeatureTableResponseBY\n\x10\x66\x65\x61st.proto.coreB\x10\x43oreServiceProtoZ3github.com/feast-dev/feast/sdk/go/protos/feast/coreb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,tensorflow__metadata_dot_proto_dot_v0_dot_statistics__pb2.DESCRIPTOR,feast_dot_core_dot_Entity__pb2.DESCRIPTOR,feast_dot_core_dot_Feature__pb2.DESCRIPTOR,feast_dot_core_dot_FeatureTable__pb2.DESCRIPTOR,feast_dot_core_dot_Store__pb2.DESCRIPTOR,])



_UPDATESTORERESPONSE_STATUS = _descriptor.EnumDescriptor(
  name='Status',
  full_name='feast.core.UpdateStoreResponse.Status',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NO_CHANGE', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UPDATED', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1578,
  serialized_end=1614,
)
_sym_db.RegisterEnumDescriptor(_UPDATESTORERESPONSE_STATUS)


_GETENTITYREQUEST = _descriptor.Descriptor(
  name='GetEntityRequest',
  full_name='feast.core.GetEntityRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='feast.core.GetEntityRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='project', full_name='feast.core.GetEntityRequest.project', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=230,
  serialized_end=279,
)


_GETENTITYRESPONSE = _descriptor.Descriptor(
  name='GetEntityResponse',
  full_name='feast.core.GetEntityResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='entity', full_name='feast.core.GetEntityResponse.entity', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=281,
  serialized_end=336,
)


_LISTENTITIESREQUEST_FILTER_LABELSENTRY = _descriptor.Descriptor(
  name='LabelsEntry',
  full_name='feast.core.ListEntitiesRequest.Filter.LabelsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='feast.core.ListEntitiesRequest.Filter.LabelsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='feast.core.ListEntitiesRequest.Filter.LabelsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=514,
  serialized_end=559,
)

_LISTENTITIESREQUEST_FILTER = _descriptor.Descriptor(
  name='Filter',
  full_name='feast.core.ListEntitiesRequest.Filter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='project', full_name='feast.core.ListEntitiesRequest.Filter.project', index=0,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='labels', full_name='feast.core.ListEntitiesRequest.Filter.labels', index=1,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_LISTENTITIESREQUEST_FILTER_LABELSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=419,
  serialized_end=559,
)

_LISTENTITIESREQUEST = _descriptor.Descriptor(
  name='ListEntitiesRequest',
  full_name='feast.core.ListEntitiesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='filter', full_name='feast.core.ListEntitiesRequest.filter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_LISTENTITIESREQUEST_FILTER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=339,
  serialized_end=559,
)


_LISTENTITIESRESPONSE = _descriptor.Descriptor(
  name='ListEntitiesResponse',
  full_name='feast.core.ListEntitiesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='entities', full_name='feast.core.ListEntitiesResponse.entities', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=561,
  serialized_end=621,
)


_LISTFEATURESREQUEST_FILTER_LABELSENTRY = _descriptor.Descriptor(
  name='LabelsEntry',
  full_name='feast.core.ListFeaturesRequest.Filter.LabelsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='feast.core.ListFeaturesRequest.Filter.LabelsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='feast.core.ListFeaturesRequest.Filter.LabelsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=514,
  serialized_end=559,
)

_LISTFEATURESREQUEST_FILTER = _descriptor.Descriptor(
  name='Filter',
  full_name='feast.core.ListFeaturesRequest.Filter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='labels', full_name='feast.core.ListFeaturesRequest.Filter.labels', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='entities', full_name='feast.core.ListFeaturesRequest.Filter.entities', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='project', full_name='feast.core.ListFeaturesRequest.Filter.project', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_LISTFEATURESREQUEST_FILTER_LABELSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=704,
  serialized_end=862,
)

_LISTFEATURESREQUEST = _descriptor.Descriptor(
  name='ListFeaturesRequest',
  full_name='feast.core.ListFeaturesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='filter', full_name='feast.core.ListFeaturesRequest.filter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_LISTFEATURESREQUEST_FILTER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=624,
  serialized_end=862,
)


_LISTFEATURESRESPONSE_FEATURESENTRY = _descriptor.Descriptor(
  name='FeaturesEntry',
  full_name='feast.core.ListFeaturesResponse.FeaturesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='feast.core.ListFeaturesResponse.FeaturesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='feast.core.ListFeaturesResponse.FeaturesEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=955,
  serialized_end=1029,
)

_LISTFEATURESRESPONSE = _descriptor.Descriptor(
  name='ListFeaturesResponse',
  full_name='feast.core.ListFeaturesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='features', full_name='feast.core.ListFeaturesResponse.features', index=0,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_LISTFEATURESRESPONSE_FEATURESENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=865,
  serialized_end=1035,
)


_LISTSTORESREQUEST_FILTER = _descriptor.Descriptor(
  name='Filter',
  full_name='feast.core.ListStoresRequest.Filter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='feast.core.ListStoresRequest.Filter.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1112,
  serialized_end=1134,
)

_LISTSTORESREQUEST = _descriptor.Descriptor(
  name='ListStoresRequest',
  full_name='feast.core.ListStoresRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='filter', full_name='feast.core.ListStoresRequest.filter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_LISTSTORESREQUEST_FILTER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1037,
  serialized_end=1134,
)


_LISTSTORESRESPONSE = _descriptor.Descriptor(
  name='ListStoresResponse',
  full_name='feast.core.ListStoresResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='store', full_name='feast.core.ListStoresResponse.store', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1136,
  serialized_end=1190,
)


_APPLYENTITYREQUEST = _descriptor.Descriptor(
  name='ApplyEntityRequest',
  full_name='feast.core.ApplyEntityRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='spec', full_name='feast.core.ApplyEntityRequest.spec', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='project', full_name='feast.core.ApplyEntityRequest.project', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1192,
  serialized_end=1269,
)


_APPLYENTITYRESPONSE = _descriptor.Descriptor(
  name='ApplyEntityResponse',
  full_name='feast.core.ApplyEntityResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='entity', full_name='feast.core.ApplyEntityResponse.entity', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1271,
  serialized_end=1328,
)


_GETFEASTCOREVERSIONREQUEST = _descriptor.Descriptor(
  name='GetFeastCoreVersionRequest',
  full_name='feast.core.GetFeastCoreVersionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1330,
  serialized_end=1358,
)


_GETFEASTCOREVERSIONRESPONSE = _descriptor.Descriptor(
  name='GetFeastCoreVersionResponse',
  full_name='feast.core.GetFeastCoreVersionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='feast.core.GetFeastCoreVersionResponse.version', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1360,
  serialized_end=1406,
)


_UPDATESTOREREQUEST = _descriptor.Descriptor(
  name='UpdateStoreRequest',
  full_name='feast.core.UpdateStoreRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='store', full_name='feast.core.UpdateStoreRequest.store', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1408,
  serialized_end=1462,
)


_UPDATESTORERESPONSE = _descriptor.Descriptor(
  name='UpdateStoreResponse',
  full_name='feast.core.UpdateStoreResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='store', full_name='feast.core.UpdateStoreResponse.store', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='status', full_name='feast.core.UpdateStoreResponse.status', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _UPDATESTORERESPONSE_STATUS,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1465,
  serialized_end=1614,
)


_CREATEPROJECTREQUEST = _descriptor.Descriptor(
  name='CreateProjectRequest',
  full_name='feast.core.CreateProjectRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='feast.core.CreateProjectRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1616,
  serialized_end=1652,
)


_CREATEPROJECTRESPONSE = _descriptor.Descriptor(
  name='CreateProjectResponse',
  full_name='feast.core.CreateProjectResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1654,
  serialized_end=1677,
)


_ARCHIVEPROJECTREQUEST = _descriptor.Descriptor(
  name='ArchiveProjectRequest',
  full_name='feast.core.ArchiveProjectRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='feast.core.ArchiveProjectRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1679,
  serialized_end=1716,
)


_ARCHIVEPROJECTRESPONSE = _descriptor.Descriptor(
  name='ArchiveProjectResponse',
  full_name='feast.core.ArchiveProjectResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1718,
  serialized_end=1742,
)


_LISTPROJECTSREQUEST = _descriptor.Descriptor(
  name='ListProjectsRequest',
  full_name='feast.core.ListProjectsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1744,
  serialized_end=1765,
)


_LISTPROJECTSRESPONSE = _descriptor.Descriptor(
  name='ListProjectsResponse',
  full_name='feast.core.ListProjectsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='projects', full_name='feast.core.ListProjectsResponse.projects', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1767,
  serialized_end=1807,
)


_UPDATEFEATURESETSTATUSRESPONSE = _descriptor.Descriptor(
  name='UpdateFeatureSetStatusResponse',
  full_name='feast.core.UpdateFeatureSetStatusResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1809,
  serialized_end=1841,
)


_APPLYFEATURETABLEREQUEST = _descriptor.Descriptor(
  name='ApplyFeatureTableRequest',
  full_name='feast.core.ApplyFeatureTableRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='project', full_name='feast.core.ApplyFeatureTableRequest.project', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='table_spec', full_name='feast.core.ApplyFeatureTableRequest.table_spec', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1843,
  serialized_end=1936,
)


_APPLYFEATURETABLERESPONSE = _descriptor.Descriptor(
  name='ApplyFeatureTableResponse',
  full_name='feast.core.ApplyFeatureTableResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='table', full_name='feast.core.ApplyFeatureTableResponse.table', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1938,
  serialized_end=2006,
)


_GETFEATURETABLEREQUEST = _descriptor.Descriptor(
  name='GetFeatureTableRequest',
  full_name='feast.core.GetFeatureTableRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='project', full_name='feast.core.GetFeatureTableRequest.project', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='feast.core.GetFeatureTableRequest.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2008,
  serialized_end=2063,
)


_GETFEATURETABLERESPONSE = _descriptor.Descriptor(
  name='GetFeatureTableResponse',
  full_name='feast.core.GetFeatureTableResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='table', full_name='feast.core.GetFeatureTableResponse.table', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2065,
  serialized_end=2131,
)


_LISTFEATURETABLESREQUEST_FILTER_LABELSENTRY = _descriptor.Descriptor(
  name='LabelsEntry',
  full_name='feast.core.ListFeatureTablesRequest.Filter.LabelsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='feast.core.ListFeatureTablesRequest.Filter.LabelsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='feast.core.ListFeatureTablesRequest.Filter.LabelsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=514,
  serialized_end=559,
)

_LISTFEATURETABLESREQUEST_FILTER = _descriptor.Descriptor(
  name='Filter',
  full_name='feast.core.ListFeatureTablesRequest.Filter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='project', full_name='feast.core.ListFeatureTablesRequest.Filter.project', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='labels', full_name='feast.core.ListFeatureTablesRequest.Filter.labels', index=1,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_LISTFEATURETABLESREQUEST_FILTER_LABELSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2224,
  serialized_end=2369,
)

_LISTFEATURETABLESREQUEST = _descriptor.Descriptor(
  name='ListFeatureTablesRequest',
  full_name='feast.core.ListFeatureTablesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='filter', full_name='feast.core.ListFeatureTablesRequest.filter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_LISTFEATURETABLESREQUEST_FILTER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2134,
  serialized_end=2369,
)


_LISTFEATURETABLESRESPONSE = _descriptor.Descriptor(
  name='ListFeatureTablesResponse',
  full_name='feast.core.ListFeatureTablesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tables', full_name='feast.core.ListFeatureTablesResponse.tables', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2371,
  serialized_end=2440,
)


_DELETEFEATURETABLEREQUEST = _descriptor.Descriptor(
  name='DeleteFeatureTableRequest',
  full_name='feast.core.DeleteFeatureTableRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='project', full_name='feast.core.DeleteFeatureTableRequest.project', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='feast.core.DeleteFeatureTableRequest.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2442,
  serialized_end=2500,
)


_DELETEFEATURETABLERESPONSE = _descriptor.Descriptor(
  name='DeleteFeatureTableResponse',
  full_name='feast.core.DeleteFeatureTableResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2502,
  serialized_end=2530,
)

_GETENTITYRESPONSE.fields_by_name['entity'].message_type = feast_dot_core_dot_Entity__pb2._ENTITY
_LISTENTITIESREQUEST_FILTER_LABELSENTRY.containing_type = _LISTENTITIESREQUEST_FILTER
_LISTENTITIESREQUEST_FILTER.fields_by_name['labels'].message_type = _LISTENTITIESREQUEST_FILTER_LABELSENTRY
_LISTENTITIESREQUEST_FILTER.containing_type = _LISTENTITIESREQUEST
_LISTENTITIESREQUEST.fields_by_name['filter'].message_type = _LISTENTITIESREQUEST_FILTER
_LISTENTITIESRESPONSE.fields_by_name['entities'].message_type = feast_dot_core_dot_Entity__pb2._ENTITY
_LISTFEATURESREQUEST_FILTER_LABELSENTRY.containing_type = _LISTFEATURESREQUEST_FILTER
_LISTFEATURESREQUEST_FILTER.fields_by_name['labels'].message_type = _LISTFEATURESREQUEST_FILTER_LABELSENTRY
_LISTFEATURESREQUEST_FILTER.containing_type = _LISTFEATURESREQUEST
_LISTFEATURESREQUEST.fields_by_name['filter'].message_type = _LISTFEATURESREQUEST_FILTER
_LISTFEATURESRESPONSE_FEATURESENTRY.fields_by_name['value'].message_type = feast_dot_core_dot_Feature__pb2._FEATURESPECV2
_LISTFEATURESRESPONSE_FEATURESENTRY.containing_type = _LISTFEATURESRESPONSE
_LISTFEATURESRESPONSE.fields_by_name['features'].message_type = _LISTFEATURESRESPONSE_FEATURESENTRY
_LISTSTORESREQUEST_FILTER.containing_type = _LISTSTORESREQUEST
_LISTSTORESREQUEST.fields_by_name['filter'].message_type = _LISTSTORESREQUEST_FILTER
_LISTSTORESRESPONSE.fields_by_name['store'].message_type = feast_dot_core_dot_Store__pb2._STORE
_APPLYENTITYREQUEST.fields_by_name['spec'].message_type = feast_dot_core_dot_Entity__pb2._ENTITYSPECV2
_APPLYENTITYRESPONSE.fields_by_name['entity'].message_type = feast_dot_core_dot_Entity__pb2._ENTITY
_UPDATESTOREREQUEST.fields_by_name['store'].message_type = feast_dot_core_dot_Store__pb2._STORE
_UPDATESTORERESPONSE.fields_by_name['store'].message_type = feast_dot_core_dot_Store__pb2._STORE
_UPDATESTORERESPONSE.fields_by_name['status'].enum_type = _UPDATESTORERESPONSE_STATUS
_UPDATESTORERESPONSE_STATUS.containing_type = _UPDATESTORERESPONSE
_APPLYFEATURETABLEREQUEST.fields_by_name['table_spec'].message_type = feast_dot_core_dot_FeatureTable__pb2._FEATURETABLESPEC
_APPLYFEATURETABLERESPONSE.fields_by_name['table'].message_type = feast_dot_core_dot_FeatureTable__pb2._FEATURETABLE
_GETFEATURETABLERESPONSE.fields_by_name['table'].message_type = feast_dot_core_dot_FeatureTable__pb2._FEATURETABLE
_LISTFEATURETABLESREQUEST_FILTER_LABELSENTRY.containing_type = _LISTFEATURETABLESREQUEST_FILTER
_LISTFEATURETABLESREQUEST_FILTER.fields_by_name['labels'].message_type = _LISTFEATURETABLESREQUEST_FILTER_LABELSENTRY
_LISTFEATURETABLESREQUEST_FILTER.containing_type = _LISTFEATURETABLESREQUEST
_LISTFEATURETABLESREQUEST.fields_by_name['filter'].message_type = _LISTFEATURETABLESREQUEST_FILTER
_LISTFEATURETABLESRESPONSE.fields_by_name['tables'].message_type = feast_dot_core_dot_FeatureTable__pb2._FEATURETABLE
DESCRIPTOR.message_types_by_name['GetEntityRequest'] = _GETENTITYREQUEST
DESCRIPTOR.message_types_by_name['GetEntityResponse'] = _GETENTITYRESPONSE
DESCRIPTOR.message_types_by_name['ListEntitiesRequest'] = _LISTENTITIESREQUEST
DESCRIPTOR.message_types_by_name['ListEntitiesResponse'] = _LISTENTITIESRESPONSE
DESCRIPTOR.message_types_by_name['ListFeaturesRequest'] = _LISTFEATURESREQUEST
DESCRIPTOR.message_types_by_name['ListFeaturesResponse'] = _LISTFEATURESRESPONSE
DESCRIPTOR.message_types_by_name['ListStoresRequest'] = _LISTSTORESREQUEST
DESCRIPTOR.message_types_by_name['ListStoresResponse'] = _LISTSTORESRESPONSE
DESCRIPTOR.message_types_by_name['ApplyEntityRequest'] = _APPLYENTITYREQUEST
DESCRIPTOR.message_types_by_name['ApplyEntityResponse'] = _APPLYENTITYRESPONSE
DESCRIPTOR.message_types_by_name['GetFeastCoreVersionRequest'] = _GETFEASTCOREVERSIONREQUEST
DESCRIPTOR.message_types_by_name['GetFeastCoreVersionResponse'] = _GETFEASTCOREVERSIONRESPONSE
DESCRIPTOR.message_types_by_name['UpdateStoreRequest'] = _UPDATESTOREREQUEST
DESCRIPTOR.message_types_by_name['UpdateStoreResponse'] = _UPDATESTORERESPONSE
DESCRIPTOR.message_types_by_name['CreateProjectRequest'] = _CREATEPROJECTREQUEST
DESCRIPTOR.message_types_by_name['CreateProjectResponse'] = _CREATEPROJECTRESPONSE
DESCRIPTOR.message_types_by_name['ArchiveProjectRequest'] = _ARCHIVEPROJECTREQUEST
DESCRIPTOR.message_types_by_name['ArchiveProjectResponse'] = _ARCHIVEPROJECTRESPONSE
DESCRIPTOR.message_types_by_name['ListProjectsRequest'] = _LISTPROJECTSREQUEST
DESCRIPTOR.message_types_by_name['ListProjectsResponse'] = _LISTPROJECTSRESPONSE
DESCRIPTOR.message_types_by_name['UpdateFeatureSetStatusResponse'] = _UPDATEFEATURESETSTATUSRESPONSE
DESCRIPTOR.message_types_by_name['ApplyFeatureTableRequest'] = _APPLYFEATURETABLEREQUEST
DESCRIPTOR.message_types_by_name['ApplyFeatureTableResponse'] = _APPLYFEATURETABLERESPONSE
DESCRIPTOR.message_types_by_name['GetFeatureTableRequest'] = _GETFEATURETABLEREQUEST
DESCRIPTOR.message_types_by_name['GetFeatureTableResponse'] = _GETFEATURETABLERESPONSE
DESCRIPTOR.message_types_by_name['ListFeatureTablesRequest'] = _LISTFEATURETABLESREQUEST
DESCRIPTOR.message_types_by_name['ListFeatureTablesResponse'] = _LISTFEATURETABLESRESPONSE
DESCRIPTOR.message_types_by_name['DeleteFeatureTableRequest'] = _DELETEFEATURETABLEREQUEST
DESCRIPTOR.message_types_by_name['DeleteFeatureTableResponse'] = _DELETEFEATURETABLERESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GetEntityRequest = _reflection.GeneratedProtocolMessageType('GetEntityRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETENTITYREQUEST,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.GetEntityRequest)
  })
_sym_db.RegisterMessage(GetEntityRequest)

GetEntityResponse = _reflection.GeneratedProtocolMessageType('GetEntityResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETENTITYRESPONSE,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.GetEntityResponse)
  })
_sym_db.RegisterMessage(GetEntityResponse)

ListEntitiesRequest = _reflection.GeneratedProtocolMessageType('ListEntitiesRequest', (_message.Message,), {

  'Filter' : _reflection.GeneratedProtocolMessageType('Filter', (_message.Message,), {

    'LabelsEntry' : _reflection.GeneratedProtocolMessageType('LabelsEntry', (_message.Message,), {
      'DESCRIPTOR' : _LISTENTITIESREQUEST_FILTER_LABELSENTRY,
      '__module__' : 'feast.core.CoreService_pb2'
      # @@protoc_insertion_point(class_scope:feast.core.ListEntitiesRequest.Filter.LabelsEntry)
      })
    ,
    'DESCRIPTOR' : _LISTENTITIESREQUEST_FILTER,
    '__module__' : 'feast.core.CoreService_pb2'
    # @@protoc_insertion_point(class_scope:feast.core.ListEntitiesRequest.Filter)
    })
  ,
  'DESCRIPTOR' : _LISTENTITIESREQUEST,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.ListEntitiesRequest)
  })
_sym_db.RegisterMessage(ListEntitiesRequest)
_sym_db.RegisterMessage(ListEntitiesRequest.Filter)
_sym_db.RegisterMessage(ListEntitiesRequest.Filter.LabelsEntry)

ListEntitiesResponse = _reflection.GeneratedProtocolMessageType('ListEntitiesResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTENTITIESRESPONSE,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.ListEntitiesResponse)
  })
_sym_db.RegisterMessage(ListEntitiesResponse)

ListFeaturesRequest = _reflection.GeneratedProtocolMessageType('ListFeaturesRequest', (_message.Message,), {

  'Filter' : _reflection.GeneratedProtocolMessageType('Filter', (_message.Message,), {

    'LabelsEntry' : _reflection.GeneratedProtocolMessageType('LabelsEntry', (_message.Message,), {
      'DESCRIPTOR' : _LISTFEATURESREQUEST_FILTER_LABELSENTRY,
      '__module__' : 'feast.core.CoreService_pb2'
      # @@protoc_insertion_point(class_scope:feast.core.ListFeaturesRequest.Filter.LabelsEntry)
      })
    ,
    'DESCRIPTOR' : _LISTFEATURESREQUEST_FILTER,
    '__module__' : 'feast.core.CoreService_pb2'
    # @@protoc_insertion_point(class_scope:feast.core.ListFeaturesRequest.Filter)
    })
  ,
  'DESCRIPTOR' : _LISTFEATURESREQUEST,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.ListFeaturesRequest)
  })
_sym_db.RegisterMessage(ListFeaturesRequest)
_sym_db.RegisterMessage(ListFeaturesRequest.Filter)
_sym_db.RegisterMessage(ListFeaturesRequest.Filter.LabelsEntry)

ListFeaturesResponse = _reflection.GeneratedProtocolMessageType('ListFeaturesResponse', (_message.Message,), {

  'FeaturesEntry' : _reflection.GeneratedProtocolMessageType('FeaturesEntry', (_message.Message,), {
    'DESCRIPTOR' : _LISTFEATURESRESPONSE_FEATURESENTRY,
    '__module__' : 'feast.core.CoreService_pb2'
    # @@protoc_insertion_point(class_scope:feast.core.ListFeaturesResponse.FeaturesEntry)
    })
  ,
  'DESCRIPTOR' : _LISTFEATURESRESPONSE,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.ListFeaturesResponse)
  })
_sym_db.RegisterMessage(ListFeaturesResponse)
_sym_db.RegisterMessage(ListFeaturesResponse.FeaturesEntry)

ListStoresRequest = _reflection.GeneratedProtocolMessageType('ListStoresRequest', (_message.Message,), {

  'Filter' : _reflection.GeneratedProtocolMessageType('Filter', (_message.Message,), {
    'DESCRIPTOR' : _LISTSTORESREQUEST_FILTER,
    '__module__' : 'feast.core.CoreService_pb2'
    # @@protoc_insertion_point(class_scope:feast.core.ListStoresRequest.Filter)
    })
  ,
  'DESCRIPTOR' : _LISTSTORESREQUEST,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.ListStoresRequest)
  })
_sym_db.RegisterMessage(ListStoresRequest)
_sym_db.RegisterMessage(ListStoresRequest.Filter)

ListStoresResponse = _reflection.GeneratedProtocolMessageType('ListStoresResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTSTORESRESPONSE,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.ListStoresResponse)
  })
_sym_db.RegisterMessage(ListStoresResponse)

ApplyEntityRequest = _reflection.GeneratedProtocolMessageType('ApplyEntityRequest', (_message.Message,), {
  'DESCRIPTOR' : _APPLYENTITYREQUEST,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.ApplyEntityRequest)
  })
_sym_db.RegisterMessage(ApplyEntityRequest)

ApplyEntityResponse = _reflection.GeneratedProtocolMessageType('ApplyEntityResponse', (_message.Message,), {
  'DESCRIPTOR' : _APPLYENTITYRESPONSE,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.ApplyEntityResponse)
  })
_sym_db.RegisterMessage(ApplyEntityResponse)

GetFeastCoreVersionRequest = _reflection.GeneratedProtocolMessageType('GetFeastCoreVersionRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETFEASTCOREVERSIONREQUEST,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.GetFeastCoreVersionRequest)
  })
_sym_db.RegisterMessage(GetFeastCoreVersionRequest)

GetFeastCoreVersionResponse = _reflection.GeneratedProtocolMessageType('GetFeastCoreVersionResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETFEASTCOREVERSIONRESPONSE,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.GetFeastCoreVersionResponse)
  })
_sym_db.RegisterMessage(GetFeastCoreVersionResponse)

UpdateStoreRequest = _reflection.GeneratedProtocolMessageType('UpdateStoreRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATESTOREREQUEST,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.UpdateStoreRequest)
  })
_sym_db.RegisterMessage(UpdateStoreRequest)

UpdateStoreResponse = _reflection.GeneratedProtocolMessageType('UpdateStoreResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATESTORERESPONSE,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.UpdateStoreResponse)
  })
_sym_db.RegisterMessage(UpdateStoreResponse)

CreateProjectRequest = _reflection.GeneratedProtocolMessageType('CreateProjectRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEPROJECTREQUEST,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.CreateProjectRequest)
  })
_sym_db.RegisterMessage(CreateProjectRequest)

CreateProjectResponse = _reflection.GeneratedProtocolMessageType('CreateProjectResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATEPROJECTRESPONSE,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.CreateProjectResponse)
  })
_sym_db.RegisterMessage(CreateProjectResponse)

ArchiveProjectRequest = _reflection.GeneratedProtocolMessageType('ArchiveProjectRequest', (_message.Message,), {
  'DESCRIPTOR' : _ARCHIVEPROJECTREQUEST,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.ArchiveProjectRequest)
  })
_sym_db.RegisterMessage(ArchiveProjectRequest)

ArchiveProjectResponse = _reflection.GeneratedProtocolMessageType('ArchiveProjectResponse', (_message.Message,), {
  'DESCRIPTOR' : _ARCHIVEPROJECTRESPONSE,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.ArchiveProjectResponse)
  })
_sym_db.RegisterMessage(ArchiveProjectResponse)

ListProjectsRequest = _reflection.GeneratedProtocolMessageType('ListProjectsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTPROJECTSREQUEST,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.ListProjectsRequest)
  })
_sym_db.RegisterMessage(ListProjectsRequest)

ListProjectsResponse = _reflection.GeneratedProtocolMessageType('ListProjectsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTPROJECTSRESPONSE,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.ListProjectsResponse)
  })
_sym_db.RegisterMessage(ListProjectsResponse)

UpdateFeatureSetStatusResponse = _reflection.GeneratedProtocolMessageType('UpdateFeatureSetStatusResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEFEATURESETSTATUSRESPONSE,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.UpdateFeatureSetStatusResponse)
  })
_sym_db.RegisterMessage(UpdateFeatureSetStatusResponse)

ApplyFeatureTableRequest = _reflection.GeneratedProtocolMessageType('ApplyFeatureTableRequest', (_message.Message,), {
  'DESCRIPTOR' : _APPLYFEATURETABLEREQUEST,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.ApplyFeatureTableRequest)
  })
_sym_db.RegisterMessage(ApplyFeatureTableRequest)

ApplyFeatureTableResponse = _reflection.GeneratedProtocolMessageType('ApplyFeatureTableResponse', (_message.Message,), {
  'DESCRIPTOR' : _APPLYFEATURETABLERESPONSE,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.ApplyFeatureTableResponse)
  })
_sym_db.RegisterMessage(ApplyFeatureTableResponse)

GetFeatureTableRequest = _reflection.GeneratedProtocolMessageType('GetFeatureTableRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATURETABLEREQUEST,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.GetFeatureTableRequest)
  })
_sym_db.RegisterMessage(GetFeatureTableRequest)

GetFeatureTableResponse = _reflection.GeneratedProtocolMessageType('GetFeatureTableResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATURETABLERESPONSE,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.GetFeatureTableResponse)
  })
_sym_db.RegisterMessage(GetFeatureTableResponse)

ListFeatureTablesRequest = _reflection.GeneratedProtocolMessageType('ListFeatureTablesRequest', (_message.Message,), {

  'Filter' : _reflection.GeneratedProtocolMessageType('Filter', (_message.Message,), {

    'LabelsEntry' : _reflection.GeneratedProtocolMessageType('LabelsEntry', (_message.Message,), {
      'DESCRIPTOR' : _LISTFEATURETABLESREQUEST_FILTER_LABELSENTRY,
      '__module__' : 'feast.core.CoreService_pb2'
      # @@protoc_insertion_point(class_scope:feast.core.ListFeatureTablesRequest.Filter.LabelsEntry)
      })
    ,
    'DESCRIPTOR' : _LISTFEATURETABLESREQUEST_FILTER,
    '__module__' : 'feast.core.CoreService_pb2'
    # @@protoc_insertion_point(class_scope:feast.core.ListFeatureTablesRequest.Filter)
    })
  ,
  'DESCRIPTOR' : _LISTFEATURETABLESREQUEST,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.ListFeatureTablesRequest)
  })
_sym_db.RegisterMessage(ListFeatureTablesRequest)
_sym_db.RegisterMessage(ListFeatureTablesRequest.Filter)
_sym_db.RegisterMessage(ListFeatureTablesRequest.Filter.LabelsEntry)

ListFeatureTablesResponse = _reflection.GeneratedProtocolMessageType('ListFeatureTablesResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTFEATURETABLESRESPONSE,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.ListFeatureTablesResponse)
  })
_sym_db.RegisterMessage(ListFeatureTablesResponse)

DeleteFeatureTableRequest = _reflection.GeneratedProtocolMessageType('DeleteFeatureTableRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEFEATURETABLEREQUEST,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.DeleteFeatureTableRequest)
  })
_sym_db.RegisterMessage(DeleteFeatureTableRequest)

DeleteFeatureTableResponse = _reflection.GeneratedProtocolMessageType('DeleteFeatureTableResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEFEATURETABLERESPONSE,
  '__module__' : 'feast.core.CoreService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.DeleteFeatureTableResponse)
  })
_sym_db.RegisterMessage(DeleteFeatureTableResponse)


DESCRIPTOR._options = None
_LISTENTITIESREQUEST_FILTER_LABELSENTRY._options = None
_LISTFEATURESREQUEST_FILTER_LABELSENTRY._options = None
_LISTFEATURESRESPONSE_FEATURESENTRY._options = None
_LISTFEATURETABLESREQUEST_FILTER_LABELSENTRY._options = None

_CORESERVICE = _descriptor.ServiceDescriptor(
  name='CoreService',
  full_name='feast.core.CoreService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=2533,
  serialized_end=3774,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetFeastCoreVersion',
    full_name='feast.core.CoreService.GetFeastCoreVersion',
    index=0,
    containing_service=None,
    input_type=_GETFEASTCOREVERSIONREQUEST,
    output_type=_GETFEASTCOREVERSIONRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetEntity',
    full_name='feast.core.CoreService.GetEntity',
    index=1,
    containing_service=None,
    input_type=_GETENTITYREQUEST,
    output_type=_GETENTITYRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListFeatures',
    full_name='feast.core.CoreService.ListFeatures',
    index=2,
    containing_service=None,
    input_type=_LISTFEATURESREQUEST,
    output_type=_LISTFEATURESRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListStores',
    full_name='feast.core.CoreService.ListStores',
    index=3,
    containing_service=None,
    input_type=_LISTSTORESREQUEST,
    output_type=_LISTSTORESRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ApplyEntity',
    full_name='feast.core.CoreService.ApplyEntity',
    index=4,
    containing_service=None,
    input_type=_APPLYENTITYREQUEST,
    output_type=_APPLYENTITYRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListEntities',
    full_name='feast.core.CoreService.ListEntities',
    index=5,
    containing_service=None,
    input_type=_LISTENTITIESREQUEST,
    output_type=_LISTENTITIESRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateStore',
    full_name='feast.core.CoreService.UpdateStore',
    index=6,
    containing_service=None,
    input_type=_UPDATESTOREREQUEST,
    output_type=_UPDATESTORERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CreateProject',
    full_name='feast.core.CoreService.CreateProject',
    index=7,
    containing_service=None,
    input_type=_CREATEPROJECTREQUEST,
    output_type=_CREATEPROJECTRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ArchiveProject',
    full_name='feast.core.CoreService.ArchiveProject',
    index=8,
    containing_service=None,
    input_type=_ARCHIVEPROJECTREQUEST,
    output_type=_ARCHIVEPROJECTRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListProjects',
    full_name='feast.core.CoreService.ListProjects',
    index=9,
    containing_service=None,
    input_type=_LISTPROJECTSREQUEST,
    output_type=_LISTPROJECTSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ApplyFeatureTable',
    full_name='feast.core.CoreService.ApplyFeatureTable',
    index=10,
    containing_service=None,
    input_type=_APPLYFEATURETABLEREQUEST,
    output_type=_APPLYFEATURETABLERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListFeatureTables',
    full_name='feast.core.CoreService.ListFeatureTables',
    index=11,
    containing_service=None,
    input_type=_LISTFEATURETABLESREQUEST,
    output_type=_LISTFEATURETABLESRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetFeatureTable',
    full_name='feast.core.CoreService.GetFeatureTable',
    index=12,
    containing_service=None,
    input_type=_GETFEATURETABLEREQUEST,
    output_type=_GETFEATURETABLERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteFeatureTable',
    full_name='feast.core.CoreService.DeleteFeatureTable',
    index=13,
    containing_service=None,
    input_type=_DELETEFEATURETABLEREQUEST,
    output_type=_DELETEFEATURETABLERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_CORESERVICE)

DESCRIPTOR.services_by_name['CoreService'] = _CORESERVICE

# @@protoc_insertion_point(module_scope)
