# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/FeatureViewProjection.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from feast.protos.feast.core import Feature_pb2 as feast_dot_core_dot_Feature__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='feast/core/FeatureViewProjection.proto',
  package='feast.core',
  syntax='proto3',
  serialized_options=b'\n\020feast.proto.coreB\025FeatureReferenceProtoZ3github.com/feast-dev/feast/sdk/go/protos/feast/core',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n&feast/core/FeatureViewProjection.proto\x12\nfeast.core\x1a\x18\x66\x65\x61st/core/Feature.proto\"\x83\x02\n\x15\x46\x65\x61tureViewProjection\x12\x19\n\x11\x66\x65\x61ture_view_name\x18\x01 \x01(\t\x12\x1f\n\x17\x66\x65\x61ture_view_name_alias\x18\x03 \x01(\t\x12\x32\n\x0f\x66\x65\x61ture_columns\x18\x02 \x03(\x0b\x32\x19.feast.core.FeatureSpecV2\x12G\n\x0cjoin_key_map\x18\x04 \x03(\x0b\x32\x31.feast.core.FeatureViewProjection.JoinKeyMapEntry\x1a\x31\n\x0fJoinKeyMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42^\n\x10\x66\x65\x61st.proto.coreB\x15\x46\x65\x61tureReferenceProtoZ3github.com/feast-dev/feast/sdk/go/protos/feast/coreb\x06proto3'
  ,
  dependencies=[feast_dot_core_dot_Feature__pb2.DESCRIPTOR,])




_FEATUREVIEWPROJECTION_JOINKEYMAPENTRY = _descriptor.Descriptor(
  name='JoinKeyMapEntry',
  full_name='feast.core.FeatureViewProjection.JoinKeyMapEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='feast.core.FeatureViewProjection.JoinKeyMapEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='feast.core.FeatureViewProjection.JoinKeyMapEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=291,
  serialized_end=340,
)

_FEATUREVIEWPROJECTION = _descriptor.Descriptor(
  name='FeatureViewProjection',
  full_name='feast.core.FeatureViewProjection',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='feature_view_name', full_name='feast.core.FeatureViewProjection.feature_view_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_view_name_alias', full_name='feast.core.FeatureViewProjection.feature_view_name_alias', index=1,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_columns', full_name='feast.core.FeatureViewProjection.feature_columns', index=2,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='join_key_map', full_name='feast.core.FeatureViewProjection.join_key_map', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_FEATUREVIEWPROJECTION_JOINKEYMAPENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=81,
  serialized_end=340,
)

_FEATUREVIEWPROJECTION_JOINKEYMAPENTRY.containing_type = _FEATUREVIEWPROJECTION
_FEATUREVIEWPROJECTION.fields_by_name['feature_columns'].message_type = feast_dot_core_dot_Feature__pb2._FEATURESPECV2
_FEATUREVIEWPROJECTION.fields_by_name['join_key_map'].message_type = _FEATUREVIEWPROJECTION_JOINKEYMAPENTRY
DESCRIPTOR.message_types_by_name['FeatureViewProjection'] = _FEATUREVIEWPROJECTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FeatureViewProjection = _reflection.GeneratedProtocolMessageType('FeatureViewProjection', (_message.Message,), {

  'JoinKeyMapEntry' : _reflection.GeneratedProtocolMessageType('JoinKeyMapEntry', (_message.Message,), {
    'DESCRIPTOR' : _FEATUREVIEWPROJECTION_JOINKEYMAPENTRY,
    '__module__' : 'feast.core.FeatureViewProjection_pb2'
    # @@protoc_insertion_point(class_scope:feast.core.FeatureViewProjection.JoinKeyMapEntry)
    })
  ,
  'DESCRIPTOR' : _FEATUREVIEWPROJECTION,
  '__module__' : 'feast.core.FeatureViewProjection_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.FeatureViewProjection)
  })
_sym_db.RegisterMessage(FeatureViewProjection)
_sym_db.RegisterMessage(FeatureViewProjection.JoinKeyMapEntry)


DESCRIPTOR._options = None
_FEATUREVIEWPROJECTION_JOINKEYMAPENTRY._options = None
# @@protoc_insertion_point(module_scope)
