# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/DataFormat.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='feast/core/DataFormat.proto',
  package='feast.core',
  syntax='proto3',
  serialized_options=b'\n\020feast.proto.coreB\017DataFormatProtoZ3github.com/feast-dev/feast/sdk/go/protos/feast/core',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1b\x66\x65\x61st/core/DataFormat.proto\x12\nfeast.core\"g\n\nFileFormat\x12>\n\x0eparquet_format\x18\x01 \x01(\x0b\x32$.feast.core.FileFormat.ParquetFormatH\x00\x1a\x0f\n\rParquetFormatB\x08\n\x06\x66ormat\"\xd8\x01\n\x0cStreamFormat\x12:\n\x0b\x61vro_format\x18\x01 \x01(\x0b\x32#.feast.core.StreamFormat.AvroFormatH\x00\x12<\n\x0cproto_format\x18\x02 \x01(\x0b\x32$.feast.core.StreamFormat.ProtoFormatH\x00\x1a!\n\x0bProtoFormat\x12\x12\n\nclass_path\x18\x01 \x01(\t\x1a!\n\nAvroFormat\x12\x13\n\x0bschema_json\x18\x01 \x01(\tB\x08\n\x06\x66ormatBX\n\x10\x66\x65\x61st.proto.coreB\x0f\x44\x61taFormatProtoZ3github.com/feast-dev/feast/sdk/go/protos/feast/coreb\x06proto3'
)




_FILEFORMAT_PARQUETFORMAT = _descriptor.Descriptor(
  name='ParquetFormat',
  full_name='feast.core.FileFormat.ParquetFormat',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=121,
  serialized_end=136,
)

_FILEFORMAT = _descriptor.Descriptor(
  name='FileFormat',
  full_name='feast.core.FileFormat',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='parquet_format', full_name='feast.core.FileFormat.parquet_format', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_FILEFORMAT_PARQUETFORMAT, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='format', full_name='feast.core.FileFormat.format',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=43,
  serialized_end=146,
)


_STREAMFORMAT_PROTOFORMAT = _descriptor.Descriptor(
  name='ProtoFormat',
  full_name='feast.core.StreamFormat.ProtoFormat',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='class_path', full_name='feast.core.StreamFormat.ProtoFormat.class_path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=287,
  serialized_end=320,
)

_STREAMFORMAT_AVROFORMAT = _descriptor.Descriptor(
  name='AvroFormat',
  full_name='feast.core.StreamFormat.AvroFormat',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='schema_json', full_name='feast.core.StreamFormat.AvroFormat.schema_json', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=322,
  serialized_end=355,
)

_STREAMFORMAT = _descriptor.Descriptor(
  name='StreamFormat',
  full_name='feast.core.StreamFormat',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='avro_format', full_name='feast.core.StreamFormat.avro_format', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='proto_format', full_name='feast.core.StreamFormat.proto_format', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_STREAMFORMAT_PROTOFORMAT, _STREAMFORMAT_AVROFORMAT, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='format', full_name='feast.core.StreamFormat.format',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=149,
  serialized_end=365,
)

_FILEFORMAT_PARQUETFORMAT.containing_type = _FILEFORMAT
_FILEFORMAT.fields_by_name['parquet_format'].message_type = _FILEFORMAT_PARQUETFORMAT
_FILEFORMAT.oneofs_by_name['format'].fields.append(
  _FILEFORMAT.fields_by_name['parquet_format'])
_FILEFORMAT.fields_by_name['parquet_format'].containing_oneof = _FILEFORMAT.oneofs_by_name['format']
_STREAMFORMAT_PROTOFORMAT.containing_type = _STREAMFORMAT
_STREAMFORMAT_AVROFORMAT.containing_type = _STREAMFORMAT
_STREAMFORMAT.fields_by_name['avro_format'].message_type = _STREAMFORMAT_AVROFORMAT
_STREAMFORMAT.fields_by_name['proto_format'].message_type = _STREAMFORMAT_PROTOFORMAT
_STREAMFORMAT.oneofs_by_name['format'].fields.append(
  _STREAMFORMAT.fields_by_name['avro_format'])
_STREAMFORMAT.fields_by_name['avro_format'].containing_oneof = _STREAMFORMAT.oneofs_by_name['format']
_STREAMFORMAT.oneofs_by_name['format'].fields.append(
  _STREAMFORMAT.fields_by_name['proto_format'])
_STREAMFORMAT.fields_by_name['proto_format'].containing_oneof = _STREAMFORMAT.oneofs_by_name['format']
DESCRIPTOR.message_types_by_name['FileFormat'] = _FILEFORMAT
DESCRIPTOR.message_types_by_name['StreamFormat'] = _STREAMFORMAT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FileFormat = _reflection.GeneratedProtocolMessageType('FileFormat', (_message.Message,), {

  'ParquetFormat' : _reflection.GeneratedProtocolMessageType('ParquetFormat', (_message.Message,), {
    'DESCRIPTOR' : _FILEFORMAT_PARQUETFORMAT,
    '__module__' : 'feast.core.DataFormat_pb2'
    # @@protoc_insertion_point(class_scope:feast.core.FileFormat.ParquetFormat)
    })
  ,
  'DESCRIPTOR' : _FILEFORMAT,
  '__module__' : 'feast.core.DataFormat_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.FileFormat)
  })
_sym_db.RegisterMessage(FileFormat)
_sym_db.RegisterMessage(FileFormat.ParquetFormat)

StreamFormat = _reflection.GeneratedProtocolMessageType('StreamFormat', (_message.Message,), {

  'ProtoFormat' : _reflection.GeneratedProtocolMessageType('ProtoFormat', (_message.Message,), {
    'DESCRIPTOR' : _STREAMFORMAT_PROTOFORMAT,
    '__module__' : 'feast.core.DataFormat_pb2'
    # @@protoc_insertion_point(class_scope:feast.core.StreamFormat.ProtoFormat)
    })
  ,

  'AvroFormat' : _reflection.GeneratedProtocolMessageType('AvroFormat', (_message.Message,), {
    'DESCRIPTOR' : _STREAMFORMAT_AVROFORMAT,
    '__module__' : 'feast.core.DataFormat_pb2'
    # @@protoc_insertion_point(class_scope:feast.core.StreamFormat.AvroFormat)
    })
  ,
  'DESCRIPTOR' : _STREAMFORMAT,
  '__module__' : 'feast.core.DataFormat_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.StreamFormat)
  })
_sym_db.RegisterMessage(StreamFormat)
_sym_db.RegisterMessage(StreamFormat.ProtoFormat)
_sym_db.RegisterMessage(StreamFormat.AvroFormat)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
