# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/types/EntityKey.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from feast.protos.feast.types import Value_pb2 as feast_dot_types_dot_Value__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='feast/types/EntityKey.proto',
  package='feast.types',
  syntax='proto3',
  serialized_options=b'\n\021feast.proto.typesB\016EntityKeyProtoZ4github.com/feast-dev/feast/sdk/go/protos/feast/types',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1b\x66\x65\x61st/types/EntityKey.proto\x12\x0b\x66\x65\x61st.types\x1a\x17\x66\x65\x61st/types/Value.proto\"I\n\tEntityKey\x12\x11\n\tjoin_keys\x18\x01 \x03(\t\x12)\n\rentity_values\x18\x02 \x03(\x0b\x32\x12.feast.types.ValueBY\n\x11\x66\x65\x61st.proto.typesB\x0e\x45ntityKeyProtoZ4github.com/feast-dev/feast/sdk/go/protos/feast/typesb\x06proto3'
  ,
  dependencies=[feast_dot_types_dot_Value__pb2.DESCRIPTOR,])




_ENTITYKEY = _descriptor.Descriptor(
  name='EntityKey',
  full_name='feast.types.EntityKey',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='join_keys', full_name='feast.types.EntityKey.join_keys', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='entity_values', full_name='feast.types.EntityKey.entity_values', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=69,
  serialized_end=142,
)

_ENTITYKEY.fields_by_name['entity_values'].message_type = feast_dot_types_dot_Value__pb2._VALUE
DESCRIPTOR.message_types_by_name['EntityKey'] = _ENTITYKEY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

EntityKey = _reflection.GeneratedProtocolMessageType('EntityKey', (_message.Message,), {
  'DESCRIPTOR' : _ENTITYKEY,
  '__module__' : 'feast.types.EntityKey_pb2'
  # @@protoc_insertion_point(class_scope:feast.types.EntityKey)
  })
_sym_db.RegisterMessage(EntityKey)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
