# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from feast.protos.feast.serving import ServingService_pb2 as feast_dot_serving_dot_ServingService__pb2


class ServingServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetFeastServingInfo = channel.unary_unary(
                '/feast.serving.ServingService/GetFeastServingInfo',
                request_serializer=feast_dot_serving_dot_ServingService__pb2.GetFeastServingInfoRequest.SerializeToString,
                response_deserializer=feast_dot_serving_dot_ServingService__pb2.GetFeastServingInfoResponse.FromString,
                )
        self.GetOnlineFeaturesV2 = channel.unary_unary(
                '/feast.serving.ServingService/GetOnlineFeaturesV2',
                request_serializer=feast_dot_serving_dot_ServingService__pb2.GetOnlineFeaturesRequestV2.SerializeToString,
                response_deserializer=feast_dot_serving_dot_ServingService__pb2.GetOnlineFeaturesResponse.FromString,
                )


class ServingServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetFeastServingInfo(self, request, context):
        """Get information about this Feast serving.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetOnlineFeaturesV2(self, request, context):
        """Get online features (v2) synchronously.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ServingServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetFeastServingInfo': grpc.unary_unary_rpc_method_handler(
                    servicer.GetFeastServingInfo,
                    request_deserializer=feast_dot_serving_dot_ServingService__pb2.GetFeastServingInfoRequest.FromString,
                    response_serializer=feast_dot_serving_dot_ServingService__pb2.GetFeastServingInfoResponse.SerializeToString,
            ),
            'GetOnlineFeaturesV2': grpc.unary_unary_rpc_method_handler(
                    servicer.GetOnlineFeaturesV2,
                    request_deserializer=feast_dot_serving_dot_ServingService__pb2.GetOnlineFeaturesRequestV2.FromString,
                    response_serializer=feast_dot_serving_dot_ServingService__pb2.GetOnlineFeaturesResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'feast.serving.ServingService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class ServingService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetFeastServingInfo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.serving.ServingService/GetFeastServingInfo',
            feast_dot_serving_dot_ServingService__pb2.GetFeastServingInfoRequest.SerializeToString,
            feast_dot_serving_dot_ServingService__pb2.GetFeastServingInfoResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetOnlineFeaturesV2(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/feast.serving.ServingService/GetOnlineFeaturesV2',
            feast_dot_serving_dot_ServingService__pb2.GetOnlineFeaturesRequestV2.SerializeToString,
            feast_dot_serving_dot_ServingService__pb2.GetOnlineFeaturesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)
