# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow_metadata/proto/v0/statistics.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorflow_metadata.proto.v0 import path_pb2 as tensorflow__metadata_dot_proto_dot_v0_dot_path__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tensorflow_metadata/proto/v0/statistics.proto',
  package='tensorflow.metadata.v0',
  syntax='proto3',
  serialized_options=b'\n\032org.tensorflow.metadata.v0P\001ZEgithub.com/feast-dev/feast/sdk/go/protos/tensorflow_metadata/proto/v0\370\001\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n-tensorflow_metadata/proto/v0/statistics.proto\x12\x16tensorflow.metadata.v0\x1a\'tensorflow_metadata/proto/v0/path.proto\"b\n\x1c\x44\x61tasetFeatureStatisticsList\x12\x42\n\x08\x64\x61tasets\x18\x01 \x03(\x0b\x32\x30.tensorflow.metadata.v0.DatasetFeatureStatistics\"\xe6\x01\n\x18\x44\x61tasetFeatureStatistics\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x14\n\x0cnum_examples\x18\x02 \x01(\x04\x12\x1d\n\x15weighted_num_examples\x18\x04 \x01(\x01\x12?\n\x08\x66\x65\x61tures\x18\x03 \x03(\x0b\x32-.tensorflow.metadata.v0.FeatureNameStatistics\x12\x46\n\x0e\x63ross_features\x18\x05 \x03(\x0b\x32..tensorflow.metadata.v0.CrossFeatureStatistics\"\xb4\x02\n\x16\x43rossFeatureStatistics\x12,\n\x06path_x\x18\x01 \x01(\x0b\x32\x1c.tensorflow.metadata.v0.Path\x12,\n\x06path_y\x18\x02 \x01(\x0b\x32\x1c.tensorflow.metadata.v0.Path\x12\r\n\x05\x63ount\x18\x03 \x01(\x04\x12I\n\x0fnum_cross_stats\x18\x04 \x01(\x0b\x32..tensorflow.metadata.v0.NumericCrossStatisticsH\x00\x12U\n\x17\x63\x61tegorical_cross_stats\x18\x05 \x01(\x0b\x32\x32.tensorflow.metadata.v0.CategoricalCrossStatisticsH\x00\x42\r\n\x0b\x63ross_stats\"A\n\x16NumericCrossStatistics\x12\x13\n\x0b\x63orrelation\x18\x01 \x01(\x02\x12\x12\n\ncovariance\x18\x02 \x01(\x02\"R\n\x1a\x43\x61tegoricalCrossStatistics\x12\x34\n\x04lift\x18\x01 \x01(\x0b\x32&.tensorflow.metadata.v0.LiftStatistics\"\x8b\x01\n\x0eLiftStatistics\x12\x37\n\x0blift_series\x18\x01 \x03(\x0b\x32\".tensorflow.metadata.v0.LiftSeries\x12@\n\x14weighted_lift_series\x18\x02 \x03(\x0b\x32\".tensorflow.metadata.v0.LiftSeries\"\x8d\x04\n\nLiftSeries\x12\x0f\n\x05y_int\x18\x01 \x01(\x05H\x00\x12\x12\n\x08y_string\x18\x02 \x01(\tH\x00\x12=\n\x08y_bucket\x18\x03 \x01(\x0b\x32).tensorflow.metadata.v0.LiftSeries.BucketH\x00\x12\x11\n\x07y_count\x18\x04 \x01(\x04H\x01\x12\x1a\n\x10weighted_y_count\x18\x05 \x01(\x01H\x01\x12\x41\n\x0blift_values\x18\x06 \x03(\x0b\x32,.tensorflow.metadata.v0.LiftSeries.LiftValue\x1a/\n\x06\x42ucket\x12\x11\n\tlow_value\x18\x01 \x01(\x01\x12\x12\n\nhigh_value\x18\x02 \x01(\x01\x1a\xdb\x01\n\tLiftValue\x12\x0f\n\x05x_int\x18\x01 \x01(\x05H\x00\x12\x12\n\x08x_string\x18\x02 \x01(\tH\x00\x12\x0c\n\x04lift\x18\x03 \x01(\x01\x12\x11\n\x07x_count\x18\x04 \x01(\x04H\x01\x12\x1a\n\x10weighted_x_count\x18\x05 \x01(\x01H\x01\x12\x17\n\rx_and_y_count\x18\x06 \x01(\x04H\x02\x12 \n\x16weighted_x_and_y_count\x18\x07 \x01(\x01H\x02\x42\t\n\x07x_valueB\x0f\n\rx_count_valueB\x15\n\x13x_and_y_count_valueB\t\n\x07y_valueB\x0f\n\ry_count_value\"\xae\x04\n\x15\x46\x65\x61tureNameStatistics\x12\x0e\n\x04name\x18\x01 \x01(\tH\x00\x12,\n\x04path\x18\x08 \x01(\x0b\x32\x1c.tensorflow.metadata.v0.PathH\x00\x12@\n\x04type\x18\x02 \x01(\x0e\x32\x32.tensorflow.metadata.v0.FeatureNameStatistics.Type\x12>\n\tnum_stats\x18\x03 \x01(\x0b\x32).tensorflow.metadata.v0.NumericStatisticsH\x01\x12@\n\x0cstring_stats\x18\x04 \x01(\x0b\x32(.tensorflow.metadata.v0.StringStatisticsH\x01\x12>\n\x0b\x62ytes_stats\x18\x05 \x01(\x0b\x32\'.tensorflow.metadata.v0.BytesStatisticsH\x01\x12@\n\x0cstruct_stats\x18\x07 \x01(\x0b\x32(.tensorflow.metadata.v0.StructStatisticsH\x01\x12=\n\x0c\x63ustom_stats\x18\x06 \x03(\x0b\x32\'.tensorflow.metadata.v0.CustomStatistic\"=\n\x04Type\x12\x07\n\x03INT\x10\x00\x12\t\n\x05\x46LOAT\x10\x01\x12\n\n\x06STRING\x10\x02\x12\t\n\x05\x42YTES\x10\x03\x12\n\n\x06STRUCT\x10\x04\x42\n\n\x08\x66ield_idB\x07\n\x05stats\"x\n\x18WeightedCommonStatistics\x12\x17\n\x0fnum_non_missing\x18\x01 \x01(\x01\x12\x13\n\x0bnum_missing\x18\x02 \x01(\x01\x12\x16\n\x0e\x61vg_num_values\x18\x03 \x01(\x01\x12\x16\n\x0etot_num_values\x18\x04 \x01(\x01\"\xbd\x01\n\x0f\x43ustomStatistic\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x03num\x18\x02 \x01(\x01H\x00\x12\r\n\x03str\x18\x03 \x01(\tH\x00\x12\x36\n\thistogram\x18\x04 \x01(\x0b\x32!.tensorflow.metadata.v0.HistogramH\x00\x12?\n\x0erank_histogram\x18\x05 \x01(\x0b\x32%.tensorflow.metadata.v0.RankHistogramH\x00\x42\x05\n\x03val\"\xb9\x02\n\x11NumericStatistics\x12>\n\x0c\x63ommon_stats\x18\x01 \x01(\x0b\x32(.tensorflow.metadata.v0.CommonStatistics\x12\x0c\n\x04mean\x18\x02 \x01(\x01\x12\x0f\n\x07std_dev\x18\x03 \x01(\x01\x12\x11\n\tnum_zeros\x18\x04 \x01(\x04\x12\x0b\n\x03min\x18\x05 \x01(\x01\x12\x0e\n\x06median\x18\x06 \x01(\x01\x12\x0b\n\x03max\x18\x07 \x01(\x01\x12\x35\n\nhistograms\x18\x08 \x03(\x0b\x32!.tensorflow.metadata.v0.Histogram\x12Q\n\x16weighted_numeric_stats\x18\t \x01(\x0b\x32\x31.tensorflow.metadata.v0.WeightedNumericStatistics\"\xa2\x03\n\x10StringStatistics\x12>\n\x0c\x63ommon_stats\x18\x01 \x01(\x0b\x32(.tensorflow.metadata.v0.CommonStatistics\x12\x0e\n\x06unique\x18\x02 \x01(\x04\x12I\n\ntop_values\x18\x03 \x03(\x0b\x32\x35.tensorflow.metadata.v0.StringStatistics.FreqAndValue\x12\x12\n\navg_length\x18\x04 \x01(\x02\x12=\n\x0erank_histogram\x18\x05 \x01(\x0b\x32%.tensorflow.metadata.v0.RankHistogram\x12O\n\x15weighted_string_stats\x18\x06 \x01(\x0b\x32\x30.tensorflow.metadata.v0.WeightedStringStatistics\x12\x17\n\x0fvocabulary_file\x18\x07 \x01(\t\x1a\x36\n\x0c\x46reqAndValue\x12\r\n\x05value\x18\x02 \x01(\t\x12\x11\n\tfrequency\x18\x03 \x01(\x01J\x04\x08\x01\x10\x02\"\x81\x01\n\x19WeightedNumericStatistics\x12\x0c\n\x04mean\x18\x01 \x01(\x01\x12\x0f\n\x07std_dev\x18\x02 \x01(\x01\x12\x0e\n\x06median\x18\x03 \x01(\x01\x12\x35\n\nhistograms\x18\x04 \x03(\x0b\x32!.tensorflow.metadata.v0.Histogram\"\xa4\x01\n\x18WeightedStringStatistics\x12I\n\ntop_values\x18\x01 \x03(\x0b\x32\x35.tensorflow.metadata.v0.StringStatistics.FreqAndValue\x12=\n\x0erank_histogram\x18\x02 \x01(\x0b\x32%.tensorflow.metadata.v0.RankHistogram\"\xa6\x01\n\x0f\x42ytesStatistics\x12>\n\x0c\x63ommon_stats\x18\x01 \x01(\x0b\x32(.tensorflow.metadata.v0.CommonStatistics\x12\x0e\n\x06unique\x18\x02 \x01(\x04\x12\x15\n\ravg_num_bytes\x18\x03 \x01(\x02\x12\x15\n\rmin_num_bytes\x18\x04 \x01(\x02\x12\x15\n\rmax_num_bytes\x18\x05 \x01(\x02\"R\n\x10StructStatistics\x12>\n\x0c\x63ommon_stats\x18\x01 \x01(\x0b\x32(.tensorflow.metadata.v0.CommonStatistics\"\xfc\x02\n\x10\x43ommonStatistics\x12\x17\n\x0fnum_non_missing\x18\x01 \x01(\x04\x12\x13\n\x0bnum_missing\x18\x02 \x01(\x04\x12\x16\n\x0emin_num_values\x18\x03 \x01(\x04\x12\x16\n\x0emax_num_values\x18\x04 \x01(\x04\x12\x16\n\x0e\x61vg_num_values\x18\x05 \x01(\x02\x12\x16\n\x0etot_num_values\x18\x08 \x01(\x04\x12?\n\x14num_values_histogram\x18\x06 \x01(\x0b\x32!.tensorflow.metadata.v0.Histogram\x12O\n\x15weighted_common_stats\x18\x07 \x01(\x0b\x32\x30.tensorflow.metadata.v0.WeightedCommonStatistics\x12H\n\x1d\x66\x65\x61ture_list_length_histogram\x18\t \x01(\x0b\x32!.tensorflow.metadata.v0.Histogram\"\xb6\x02\n\tHistogram\x12\x0f\n\x07num_nan\x18\x01 \x01(\x04\x12\x15\n\rnum_undefined\x18\x02 \x01(\x04\x12\x39\n\x07\x62uckets\x18\x03 \x03(\x0b\x32(.tensorflow.metadata.v0.Histogram.Bucket\x12=\n\x04type\x18\x04 \x01(\x0e\x32/.tensorflow.metadata.v0.Histogram.HistogramType\x12\x0c\n\x04name\x18\x05 \x01(\t\x1aK\n\x06\x42ucket\x12\x11\n\tlow_value\x18\x01 \x01(\x01\x12\x12\n\nhigh_value\x18\x02 \x01(\x01\x12\x14\n\x0csample_count\x18\x04 \x01(\x01J\x04\x08\x03\x10\x04\",\n\rHistogramType\x12\x0c\n\x08STANDARD\x10\x00\x12\r\n\tQUANTILES\x10\x01\"\xb6\x01\n\rRankHistogram\x12=\n\x07\x62uckets\x18\x01 \x03(\x0b\x32,.tensorflow.metadata.v0.RankHistogram.Bucket\x12\x0c\n\x04name\x18\x02 \x01(\t\x1aX\n\x06\x42ucket\x12\x10\n\x08low_rank\x18\x01 \x01(\x04\x12\x11\n\thigh_rank\x18\x02 \x01(\x04\x12\r\n\x05label\x18\x04 \x01(\t\x12\x14\n\x0csample_count\x18\x05 \x01(\x01J\x04\x08\x03\x10\x04\x42h\n\x1aorg.tensorflow.metadata.v0P\x01ZEgithub.com/feast-dev/feast/sdk/go/protos/tensorflow_metadata/proto/v0\xf8\x01\x01\x62\x06proto3'
  ,
  dependencies=[tensorflow__metadata_dot_proto_dot_v0_dot_path__pb2.DESCRIPTOR,])



_FEATURENAMESTATISTICS_TYPE = _descriptor.EnumDescriptor(
  name='Type',
  full_name='tensorflow.metadata.v0.FeatureNameStatistics.Type',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='INT', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FLOAT', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='STRING', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BYTES', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='STRUCT', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2056,
  serialized_end=2117,
)
_sym_db.RegisterEnumDescriptor(_FEATURENAMESTATISTICS_TYPE)

_HISTOGRAM_HISTOGRAMTYPE = _descriptor.EnumDescriptor(
  name='HistogramType',
  full_name='tensorflow.metadata.v0.Histogram.HistogramType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='STANDARD', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='QUANTILES', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=4393,
  serialized_end=4437,
)
_sym_db.RegisterEnumDescriptor(_HISTOGRAM_HISTOGRAMTYPE)


_DATASETFEATURESTATISTICSLIST = _descriptor.Descriptor(
  name='DatasetFeatureStatisticsList',
  full_name='tensorflow.metadata.v0.DatasetFeatureStatisticsList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='datasets', full_name='tensorflow.metadata.v0.DatasetFeatureStatisticsList.datasets', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=114,
  serialized_end=212,
)


_DATASETFEATURESTATISTICS = _descriptor.Descriptor(
  name='DatasetFeatureStatistics',
  full_name='tensorflow.metadata.v0.DatasetFeatureStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='tensorflow.metadata.v0.DatasetFeatureStatistics.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_examples', full_name='tensorflow.metadata.v0.DatasetFeatureStatistics.num_examples', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='weighted_num_examples', full_name='tensorflow.metadata.v0.DatasetFeatureStatistics.weighted_num_examples', index=2,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='features', full_name='tensorflow.metadata.v0.DatasetFeatureStatistics.features', index=3,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cross_features', full_name='tensorflow.metadata.v0.DatasetFeatureStatistics.cross_features', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=215,
  serialized_end=445,
)


_CROSSFEATURESTATISTICS = _descriptor.Descriptor(
  name='CrossFeatureStatistics',
  full_name='tensorflow.metadata.v0.CrossFeatureStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='path_x', full_name='tensorflow.metadata.v0.CrossFeatureStatistics.path_x', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='path_y', full_name='tensorflow.metadata.v0.CrossFeatureStatistics.path_y', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='count', full_name='tensorflow.metadata.v0.CrossFeatureStatistics.count', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_cross_stats', full_name='tensorflow.metadata.v0.CrossFeatureStatistics.num_cross_stats', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='categorical_cross_stats', full_name='tensorflow.metadata.v0.CrossFeatureStatistics.categorical_cross_stats', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='cross_stats', full_name='tensorflow.metadata.v0.CrossFeatureStatistics.cross_stats',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=448,
  serialized_end=756,
)


_NUMERICCROSSSTATISTICS = _descriptor.Descriptor(
  name='NumericCrossStatistics',
  full_name='tensorflow.metadata.v0.NumericCrossStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='correlation', full_name='tensorflow.metadata.v0.NumericCrossStatistics.correlation', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='covariance', full_name='tensorflow.metadata.v0.NumericCrossStatistics.covariance', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=758,
  serialized_end=823,
)


_CATEGORICALCROSSSTATISTICS = _descriptor.Descriptor(
  name='CategoricalCrossStatistics',
  full_name='tensorflow.metadata.v0.CategoricalCrossStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='lift', full_name='tensorflow.metadata.v0.CategoricalCrossStatistics.lift', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=825,
  serialized_end=907,
)


_LIFTSTATISTICS = _descriptor.Descriptor(
  name='LiftStatistics',
  full_name='tensorflow.metadata.v0.LiftStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='lift_series', full_name='tensorflow.metadata.v0.LiftStatistics.lift_series', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='weighted_lift_series', full_name='tensorflow.metadata.v0.LiftStatistics.weighted_lift_series', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=910,
  serialized_end=1049,
)


_LIFTSERIES_BUCKET = _descriptor.Descriptor(
  name='Bucket',
  full_name='tensorflow.metadata.v0.LiftSeries.Bucket',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='low_value', full_name='tensorflow.metadata.v0.LiftSeries.Bucket.low_value', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='high_value', full_name='tensorflow.metadata.v0.LiftSeries.Bucket.high_value', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1280,
  serialized_end=1327,
)

_LIFTSERIES_LIFTVALUE = _descriptor.Descriptor(
  name='LiftValue',
  full_name='tensorflow.metadata.v0.LiftSeries.LiftValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='x_int', full_name='tensorflow.metadata.v0.LiftSeries.LiftValue.x_int', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='x_string', full_name='tensorflow.metadata.v0.LiftSeries.LiftValue.x_string', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='lift', full_name='tensorflow.metadata.v0.LiftSeries.LiftValue.lift', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='x_count', full_name='tensorflow.metadata.v0.LiftSeries.LiftValue.x_count', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='weighted_x_count', full_name='tensorflow.metadata.v0.LiftSeries.LiftValue.weighted_x_count', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='x_and_y_count', full_name='tensorflow.metadata.v0.LiftSeries.LiftValue.x_and_y_count', index=5,
      number=6, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='weighted_x_and_y_count', full_name='tensorflow.metadata.v0.LiftSeries.LiftValue.weighted_x_and_y_count', index=6,
      number=7, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='x_value', full_name='tensorflow.metadata.v0.LiftSeries.LiftValue.x_value',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='x_count_value', full_name='tensorflow.metadata.v0.LiftSeries.LiftValue.x_count_value',
      index=1, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='x_and_y_count_value', full_name='tensorflow.metadata.v0.LiftSeries.LiftValue.x_and_y_count_value',
      index=2, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1330,
  serialized_end=1549,
)

_LIFTSERIES = _descriptor.Descriptor(
  name='LiftSeries',
  full_name='tensorflow.metadata.v0.LiftSeries',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='y_int', full_name='tensorflow.metadata.v0.LiftSeries.y_int', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='y_string', full_name='tensorflow.metadata.v0.LiftSeries.y_string', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='y_bucket', full_name='tensorflow.metadata.v0.LiftSeries.y_bucket', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='y_count', full_name='tensorflow.metadata.v0.LiftSeries.y_count', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='weighted_y_count', full_name='tensorflow.metadata.v0.LiftSeries.weighted_y_count', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='lift_values', full_name='tensorflow.metadata.v0.LiftSeries.lift_values', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_LIFTSERIES_BUCKET, _LIFTSERIES_LIFTVALUE, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='y_value', full_name='tensorflow.metadata.v0.LiftSeries.y_value',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='y_count_value', full_name='tensorflow.metadata.v0.LiftSeries.y_count_value',
      index=1, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1052,
  serialized_end=1577,
)


_FEATURENAMESTATISTICS = _descriptor.Descriptor(
  name='FeatureNameStatistics',
  full_name='tensorflow.metadata.v0.FeatureNameStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='tensorflow.metadata.v0.FeatureNameStatistics.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='path', full_name='tensorflow.metadata.v0.FeatureNameStatistics.path', index=1,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='tensorflow.metadata.v0.FeatureNameStatistics.type', index=2,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_stats', full_name='tensorflow.metadata.v0.FeatureNameStatistics.num_stats', index=3,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='string_stats', full_name='tensorflow.metadata.v0.FeatureNameStatistics.string_stats', index=4,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bytes_stats', full_name='tensorflow.metadata.v0.FeatureNameStatistics.bytes_stats', index=5,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='struct_stats', full_name='tensorflow.metadata.v0.FeatureNameStatistics.struct_stats', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='custom_stats', full_name='tensorflow.metadata.v0.FeatureNameStatistics.custom_stats', index=7,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _FEATURENAMESTATISTICS_TYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='field_id', full_name='tensorflow.metadata.v0.FeatureNameStatistics.field_id',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='stats', full_name='tensorflow.metadata.v0.FeatureNameStatistics.stats',
      index=1, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1580,
  serialized_end=2138,
)


_WEIGHTEDCOMMONSTATISTICS = _descriptor.Descriptor(
  name='WeightedCommonStatistics',
  full_name='tensorflow.metadata.v0.WeightedCommonStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='num_non_missing', full_name='tensorflow.metadata.v0.WeightedCommonStatistics.num_non_missing', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_missing', full_name='tensorflow.metadata.v0.WeightedCommonStatistics.num_missing', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='avg_num_values', full_name='tensorflow.metadata.v0.WeightedCommonStatistics.avg_num_values', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tot_num_values', full_name='tensorflow.metadata.v0.WeightedCommonStatistics.tot_num_values', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2140,
  serialized_end=2260,
)


_CUSTOMSTATISTIC = _descriptor.Descriptor(
  name='CustomStatistic',
  full_name='tensorflow.metadata.v0.CustomStatistic',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='tensorflow.metadata.v0.CustomStatistic.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num', full_name='tensorflow.metadata.v0.CustomStatistic.num', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='str', full_name='tensorflow.metadata.v0.CustomStatistic.str', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='histogram', full_name='tensorflow.metadata.v0.CustomStatistic.histogram', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rank_histogram', full_name='tensorflow.metadata.v0.CustomStatistic.rank_histogram', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='val', full_name='tensorflow.metadata.v0.CustomStatistic.val',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=2263,
  serialized_end=2452,
)


_NUMERICSTATISTICS = _descriptor.Descriptor(
  name='NumericStatistics',
  full_name='tensorflow.metadata.v0.NumericStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='common_stats', full_name='tensorflow.metadata.v0.NumericStatistics.common_stats', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mean', full_name='tensorflow.metadata.v0.NumericStatistics.mean', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='std_dev', full_name='tensorflow.metadata.v0.NumericStatistics.std_dev', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_zeros', full_name='tensorflow.metadata.v0.NumericStatistics.num_zeros', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='min', full_name='tensorflow.metadata.v0.NumericStatistics.min', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='median', full_name='tensorflow.metadata.v0.NumericStatistics.median', index=5,
      number=6, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max', full_name='tensorflow.metadata.v0.NumericStatistics.max', index=6,
      number=7, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='histograms', full_name='tensorflow.metadata.v0.NumericStatistics.histograms', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='weighted_numeric_stats', full_name='tensorflow.metadata.v0.NumericStatistics.weighted_numeric_stats', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2455,
  serialized_end=2768,
)


_STRINGSTATISTICS_FREQANDVALUE = _descriptor.Descriptor(
  name='FreqAndValue',
  full_name='tensorflow.metadata.v0.StringStatistics.FreqAndValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='tensorflow.metadata.v0.StringStatistics.FreqAndValue.value', index=0,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='frequency', full_name='tensorflow.metadata.v0.StringStatistics.FreqAndValue.frequency', index=1,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3135,
  serialized_end=3189,
)

_STRINGSTATISTICS = _descriptor.Descriptor(
  name='StringStatistics',
  full_name='tensorflow.metadata.v0.StringStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='common_stats', full_name='tensorflow.metadata.v0.StringStatistics.common_stats', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='unique', full_name='tensorflow.metadata.v0.StringStatistics.unique', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='top_values', full_name='tensorflow.metadata.v0.StringStatistics.top_values', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='avg_length', full_name='tensorflow.metadata.v0.StringStatistics.avg_length', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rank_histogram', full_name='tensorflow.metadata.v0.StringStatistics.rank_histogram', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='weighted_string_stats', full_name='tensorflow.metadata.v0.StringStatistics.weighted_string_stats', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='vocabulary_file', full_name='tensorflow.metadata.v0.StringStatistics.vocabulary_file', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_STRINGSTATISTICS_FREQANDVALUE, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2771,
  serialized_end=3189,
)


_WEIGHTEDNUMERICSTATISTICS = _descriptor.Descriptor(
  name='WeightedNumericStatistics',
  full_name='tensorflow.metadata.v0.WeightedNumericStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='mean', full_name='tensorflow.metadata.v0.WeightedNumericStatistics.mean', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='std_dev', full_name='tensorflow.metadata.v0.WeightedNumericStatistics.std_dev', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='median', full_name='tensorflow.metadata.v0.WeightedNumericStatistics.median', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='histograms', full_name='tensorflow.metadata.v0.WeightedNumericStatistics.histograms', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3192,
  serialized_end=3321,
)


_WEIGHTEDSTRINGSTATISTICS = _descriptor.Descriptor(
  name='WeightedStringStatistics',
  full_name='tensorflow.metadata.v0.WeightedStringStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='top_values', full_name='tensorflow.metadata.v0.WeightedStringStatistics.top_values', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rank_histogram', full_name='tensorflow.metadata.v0.WeightedStringStatistics.rank_histogram', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3324,
  serialized_end=3488,
)


_BYTESSTATISTICS = _descriptor.Descriptor(
  name='BytesStatistics',
  full_name='tensorflow.metadata.v0.BytesStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='common_stats', full_name='tensorflow.metadata.v0.BytesStatistics.common_stats', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='unique', full_name='tensorflow.metadata.v0.BytesStatistics.unique', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='avg_num_bytes', full_name='tensorflow.metadata.v0.BytesStatistics.avg_num_bytes', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='min_num_bytes', full_name='tensorflow.metadata.v0.BytesStatistics.min_num_bytes', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_num_bytes', full_name='tensorflow.metadata.v0.BytesStatistics.max_num_bytes', index=4,
      number=5, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3491,
  serialized_end=3657,
)


_STRUCTSTATISTICS = _descriptor.Descriptor(
  name='StructStatistics',
  full_name='tensorflow.metadata.v0.StructStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='common_stats', full_name='tensorflow.metadata.v0.StructStatistics.common_stats', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3659,
  serialized_end=3741,
)


_COMMONSTATISTICS = _descriptor.Descriptor(
  name='CommonStatistics',
  full_name='tensorflow.metadata.v0.CommonStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='num_non_missing', full_name='tensorflow.metadata.v0.CommonStatistics.num_non_missing', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_missing', full_name='tensorflow.metadata.v0.CommonStatistics.num_missing', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='min_num_values', full_name='tensorflow.metadata.v0.CommonStatistics.min_num_values', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_num_values', full_name='tensorflow.metadata.v0.CommonStatistics.max_num_values', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='avg_num_values', full_name='tensorflow.metadata.v0.CommonStatistics.avg_num_values', index=4,
      number=5, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tot_num_values', full_name='tensorflow.metadata.v0.CommonStatistics.tot_num_values', index=5,
      number=8, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_values_histogram', full_name='tensorflow.metadata.v0.CommonStatistics.num_values_histogram', index=6,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='weighted_common_stats', full_name='tensorflow.metadata.v0.CommonStatistics.weighted_common_stats', index=7,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_list_length_histogram', full_name='tensorflow.metadata.v0.CommonStatistics.feature_list_length_histogram', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3744,
  serialized_end=4124,
)


_HISTOGRAM_BUCKET = _descriptor.Descriptor(
  name='Bucket',
  full_name='tensorflow.metadata.v0.Histogram.Bucket',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='low_value', full_name='tensorflow.metadata.v0.Histogram.Bucket.low_value', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='high_value', full_name='tensorflow.metadata.v0.Histogram.Bucket.high_value', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sample_count', full_name='tensorflow.metadata.v0.Histogram.Bucket.sample_count', index=2,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4316,
  serialized_end=4391,
)

_HISTOGRAM = _descriptor.Descriptor(
  name='Histogram',
  full_name='tensorflow.metadata.v0.Histogram',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='num_nan', full_name='tensorflow.metadata.v0.Histogram.num_nan', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_undefined', full_name='tensorflow.metadata.v0.Histogram.num_undefined', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='buckets', full_name='tensorflow.metadata.v0.Histogram.buckets', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='tensorflow.metadata.v0.Histogram.type', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='tensorflow.metadata.v0.Histogram.name', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_HISTOGRAM_BUCKET, ],
  enum_types=[
    _HISTOGRAM_HISTOGRAMTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4127,
  serialized_end=4437,
)


_RANKHISTOGRAM_BUCKET = _descriptor.Descriptor(
  name='Bucket',
  full_name='tensorflow.metadata.v0.RankHistogram.Bucket',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='low_rank', full_name='tensorflow.metadata.v0.RankHistogram.Bucket.low_rank', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='high_rank', full_name='tensorflow.metadata.v0.RankHistogram.Bucket.high_rank', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='label', full_name='tensorflow.metadata.v0.RankHistogram.Bucket.label', index=2,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sample_count', full_name='tensorflow.metadata.v0.RankHistogram.Bucket.sample_count', index=3,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4534,
  serialized_end=4622,
)

_RANKHISTOGRAM = _descriptor.Descriptor(
  name='RankHistogram',
  full_name='tensorflow.metadata.v0.RankHistogram',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='buckets', full_name='tensorflow.metadata.v0.RankHistogram.buckets', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='tensorflow.metadata.v0.RankHistogram.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_RANKHISTOGRAM_BUCKET, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4440,
  serialized_end=4622,
)

_DATASETFEATURESTATISTICSLIST.fields_by_name['datasets'].message_type = _DATASETFEATURESTATISTICS
_DATASETFEATURESTATISTICS.fields_by_name['features'].message_type = _FEATURENAMESTATISTICS
_DATASETFEATURESTATISTICS.fields_by_name['cross_features'].message_type = _CROSSFEATURESTATISTICS
_CROSSFEATURESTATISTICS.fields_by_name['path_x'].message_type = tensorflow__metadata_dot_proto_dot_v0_dot_path__pb2._PATH
_CROSSFEATURESTATISTICS.fields_by_name['path_y'].message_type = tensorflow__metadata_dot_proto_dot_v0_dot_path__pb2._PATH
_CROSSFEATURESTATISTICS.fields_by_name['num_cross_stats'].message_type = _NUMERICCROSSSTATISTICS
_CROSSFEATURESTATISTICS.fields_by_name['categorical_cross_stats'].message_type = _CATEGORICALCROSSSTATISTICS
_CROSSFEATURESTATISTICS.oneofs_by_name['cross_stats'].fields.append(
  _CROSSFEATURESTATISTICS.fields_by_name['num_cross_stats'])
_CROSSFEATURESTATISTICS.fields_by_name['num_cross_stats'].containing_oneof = _CROSSFEATURESTATISTICS.oneofs_by_name['cross_stats']
_CROSSFEATURESTATISTICS.oneofs_by_name['cross_stats'].fields.append(
  _CROSSFEATURESTATISTICS.fields_by_name['categorical_cross_stats'])
_CROSSFEATURESTATISTICS.fields_by_name['categorical_cross_stats'].containing_oneof = _CROSSFEATURESTATISTICS.oneofs_by_name['cross_stats']
_CATEGORICALCROSSSTATISTICS.fields_by_name['lift'].message_type = _LIFTSTATISTICS
_LIFTSTATISTICS.fields_by_name['lift_series'].message_type = _LIFTSERIES
_LIFTSTATISTICS.fields_by_name['weighted_lift_series'].message_type = _LIFTSERIES
_LIFTSERIES_BUCKET.containing_type = _LIFTSERIES
_LIFTSERIES_LIFTVALUE.containing_type = _LIFTSERIES
_LIFTSERIES_LIFTVALUE.oneofs_by_name['x_value'].fields.append(
  _LIFTSERIES_LIFTVALUE.fields_by_name['x_int'])
_LIFTSERIES_LIFTVALUE.fields_by_name['x_int'].containing_oneof = _LIFTSERIES_LIFTVALUE.oneofs_by_name['x_value']
_LIFTSERIES_LIFTVALUE.oneofs_by_name['x_value'].fields.append(
  _LIFTSERIES_LIFTVALUE.fields_by_name['x_string'])
_LIFTSERIES_LIFTVALUE.fields_by_name['x_string'].containing_oneof = _LIFTSERIES_LIFTVALUE.oneofs_by_name['x_value']
_LIFTSERIES_LIFTVALUE.oneofs_by_name['x_count_value'].fields.append(
  _LIFTSERIES_LIFTVALUE.fields_by_name['x_count'])
_LIFTSERIES_LIFTVALUE.fields_by_name['x_count'].containing_oneof = _LIFTSERIES_LIFTVALUE.oneofs_by_name['x_count_value']
_LIFTSERIES_LIFTVALUE.oneofs_by_name['x_count_value'].fields.append(
  _LIFTSERIES_LIFTVALUE.fields_by_name['weighted_x_count'])
_LIFTSERIES_LIFTVALUE.fields_by_name['weighted_x_count'].containing_oneof = _LIFTSERIES_LIFTVALUE.oneofs_by_name['x_count_value']
_LIFTSERIES_LIFTVALUE.oneofs_by_name['x_and_y_count_value'].fields.append(
  _LIFTSERIES_LIFTVALUE.fields_by_name['x_and_y_count'])
_LIFTSERIES_LIFTVALUE.fields_by_name['x_and_y_count'].containing_oneof = _LIFTSERIES_LIFTVALUE.oneofs_by_name['x_and_y_count_value']
_LIFTSERIES_LIFTVALUE.oneofs_by_name['x_and_y_count_value'].fields.append(
  _LIFTSERIES_LIFTVALUE.fields_by_name['weighted_x_and_y_count'])
_LIFTSERIES_LIFTVALUE.fields_by_name['weighted_x_and_y_count'].containing_oneof = _LIFTSERIES_LIFTVALUE.oneofs_by_name['x_and_y_count_value']
_LIFTSERIES.fields_by_name['y_bucket'].message_type = _LIFTSERIES_BUCKET
_LIFTSERIES.fields_by_name['lift_values'].message_type = _LIFTSERIES_LIFTVALUE
_LIFTSERIES.oneofs_by_name['y_value'].fields.append(
  _LIFTSERIES.fields_by_name['y_int'])
_LIFTSERIES.fields_by_name['y_int'].containing_oneof = _LIFTSERIES.oneofs_by_name['y_value']
_LIFTSERIES.oneofs_by_name['y_value'].fields.append(
  _LIFTSERIES.fields_by_name['y_string'])
_LIFTSERIES.fields_by_name['y_string'].containing_oneof = _LIFTSERIES.oneofs_by_name['y_value']
_LIFTSERIES.oneofs_by_name['y_value'].fields.append(
  _LIFTSERIES.fields_by_name['y_bucket'])
_LIFTSERIES.fields_by_name['y_bucket'].containing_oneof = _LIFTSERIES.oneofs_by_name['y_value']
_LIFTSERIES.oneofs_by_name['y_count_value'].fields.append(
  _LIFTSERIES.fields_by_name['y_count'])
_LIFTSERIES.fields_by_name['y_count'].containing_oneof = _LIFTSERIES.oneofs_by_name['y_count_value']
_LIFTSERIES.oneofs_by_name['y_count_value'].fields.append(
  _LIFTSERIES.fields_by_name['weighted_y_count'])
_LIFTSERIES.fields_by_name['weighted_y_count'].containing_oneof = _LIFTSERIES.oneofs_by_name['y_count_value']
_FEATURENAMESTATISTICS.fields_by_name['path'].message_type = tensorflow__metadata_dot_proto_dot_v0_dot_path__pb2._PATH
_FEATURENAMESTATISTICS.fields_by_name['type'].enum_type = _FEATURENAMESTATISTICS_TYPE
_FEATURENAMESTATISTICS.fields_by_name['num_stats'].message_type = _NUMERICSTATISTICS
_FEATURENAMESTATISTICS.fields_by_name['string_stats'].message_type = _STRINGSTATISTICS
_FEATURENAMESTATISTICS.fields_by_name['bytes_stats'].message_type = _BYTESSTATISTICS
_FEATURENAMESTATISTICS.fields_by_name['struct_stats'].message_type = _STRUCTSTATISTICS
_FEATURENAMESTATISTICS.fields_by_name['custom_stats'].message_type = _CUSTOMSTATISTIC
_FEATURENAMESTATISTICS_TYPE.containing_type = _FEATURENAMESTATISTICS
_FEATURENAMESTATISTICS.oneofs_by_name['field_id'].fields.append(
  _FEATURENAMESTATISTICS.fields_by_name['name'])
_FEATURENAMESTATISTICS.fields_by_name['name'].containing_oneof = _FEATURENAMESTATISTICS.oneofs_by_name['field_id']
_FEATURENAMESTATISTICS.oneofs_by_name['field_id'].fields.append(
  _FEATURENAMESTATISTICS.fields_by_name['path'])
_FEATURENAMESTATISTICS.fields_by_name['path'].containing_oneof = _FEATURENAMESTATISTICS.oneofs_by_name['field_id']
_FEATURENAMESTATISTICS.oneofs_by_name['stats'].fields.append(
  _FEATURENAMESTATISTICS.fields_by_name['num_stats'])
_FEATURENAMESTATISTICS.fields_by_name['num_stats'].containing_oneof = _FEATURENAMESTATISTICS.oneofs_by_name['stats']
_FEATURENAMESTATISTICS.oneofs_by_name['stats'].fields.append(
  _FEATURENAMESTATISTICS.fields_by_name['string_stats'])
_FEATURENAMESTATISTICS.fields_by_name['string_stats'].containing_oneof = _FEATURENAMESTATISTICS.oneofs_by_name['stats']
_FEATURENAMESTATISTICS.oneofs_by_name['stats'].fields.append(
  _FEATURENAMESTATISTICS.fields_by_name['bytes_stats'])
_FEATURENAMESTATISTICS.fields_by_name['bytes_stats'].containing_oneof = _FEATURENAMESTATISTICS.oneofs_by_name['stats']
_FEATURENAMESTATISTICS.oneofs_by_name['stats'].fields.append(
  _FEATURENAMESTATISTICS.fields_by_name['struct_stats'])
_FEATURENAMESTATISTICS.fields_by_name['struct_stats'].containing_oneof = _FEATURENAMESTATISTICS.oneofs_by_name['stats']
_CUSTOMSTATISTIC.fields_by_name['histogram'].message_type = _HISTOGRAM
_CUSTOMSTATISTIC.fields_by_name['rank_histogram'].message_type = _RANKHISTOGRAM
_CUSTOMSTATISTIC.oneofs_by_name['val'].fields.append(
  _CUSTOMSTATISTIC.fields_by_name['num'])
_CUSTOMSTATISTIC.fields_by_name['num'].containing_oneof = _CUSTOMSTATISTIC.oneofs_by_name['val']
_CUSTOMSTATISTIC.oneofs_by_name['val'].fields.append(
  _CUSTOMSTATISTIC.fields_by_name['str'])
_CUSTOMSTATISTIC.fields_by_name['str'].containing_oneof = _CUSTOMSTATISTIC.oneofs_by_name['val']
_CUSTOMSTATISTIC.oneofs_by_name['val'].fields.append(
  _CUSTOMSTATISTIC.fields_by_name['histogram'])
_CUSTOMSTATISTIC.fields_by_name['histogram'].containing_oneof = _CUSTOMSTATISTIC.oneofs_by_name['val']
_CUSTOMSTATISTIC.oneofs_by_name['val'].fields.append(
  _CUSTOMSTATISTIC.fields_by_name['rank_histogram'])
_CUSTOMSTATISTIC.fields_by_name['rank_histogram'].containing_oneof = _CUSTOMSTATISTIC.oneofs_by_name['val']
_NUMERICSTATISTICS.fields_by_name['common_stats'].message_type = _COMMONSTATISTICS
_NUMERICSTATISTICS.fields_by_name['histograms'].message_type = _HISTOGRAM
_NUMERICSTATISTICS.fields_by_name['weighted_numeric_stats'].message_type = _WEIGHTEDNUMERICSTATISTICS
_STRINGSTATISTICS_FREQANDVALUE.containing_type = _STRINGSTATISTICS
_STRINGSTATISTICS.fields_by_name['common_stats'].message_type = _COMMONSTATISTICS
_STRINGSTATISTICS.fields_by_name['top_values'].message_type = _STRINGSTATISTICS_FREQANDVALUE
_STRINGSTATISTICS.fields_by_name['rank_histogram'].message_type = _RANKHISTOGRAM
_STRINGSTATISTICS.fields_by_name['weighted_string_stats'].message_type = _WEIGHTEDSTRINGSTATISTICS
_WEIGHTEDNUMERICSTATISTICS.fields_by_name['histograms'].message_type = _HISTOGRAM
_WEIGHTEDSTRINGSTATISTICS.fields_by_name['top_values'].message_type = _STRINGSTATISTICS_FREQANDVALUE
_WEIGHTEDSTRINGSTATISTICS.fields_by_name['rank_histogram'].message_type = _RANKHISTOGRAM
_BYTESSTATISTICS.fields_by_name['common_stats'].message_type = _COMMONSTATISTICS
_STRUCTSTATISTICS.fields_by_name['common_stats'].message_type = _COMMONSTATISTICS
_COMMONSTATISTICS.fields_by_name['num_values_histogram'].message_type = _HISTOGRAM
_COMMONSTATISTICS.fields_by_name['weighted_common_stats'].message_type = _WEIGHTEDCOMMONSTATISTICS
_COMMONSTATISTICS.fields_by_name['feature_list_length_histogram'].message_type = _HISTOGRAM
_HISTOGRAM_BUCKET.containing_type = _HISTOGRAM
_HISTOGRAM.fields_by_name['buckets'].message_type = _HISTOGRAM_BUCKET
_HISTOGRAM.fields_by_name['type'].enum_type = _HISTOGRAM_HISTOGRAMTYPE
_HISTOGRAM_HISTOGRAMTYPE.containing_type = _HISTOGRAM
_RANKHISTOGRAM_BUCKET.containing_type = _RANKHISTOGRAM
_RANKHISTOGRAM.fields_by_name['buckets'].message_type = _RANKHISTOGRAM_BUCKET
DESCRIPTOR.message_types_by_name['DatasetFeatureStatisticsList'] = _DATASETFEATURESTATISTICSLIST
DESCRIPTOR.message_types_by_name['DatasetFeatureStatistics'] = _DATASETFEATURESTATISTICS
DESCRIPTOR.message_types_by_name['CrossFeatureStatistics'] = _CROSSFEATURESTATISTICS
DESCRIPTOR.message_types_by_name['NumericCrossStatistics'] = _NUMERICCROSSSTATISTICS
DESCRIPTOR.message_types_by_name['CategoricalCrossStatistics'] = _CATEGORICALCROSSSTATISTICS
DESCRIPTOR.message_types_by_name['LiftStatistics'] = _LIFTSTATISTICS
DESCRIPTOR.message_types_by_name['LiftSeries'] = _LIFTSERIES
DESCRIPTOR.message_types_by_name['FeatureNameStatistics'] = _FEATURENAMESTATISTICS
DESCRIPTOR.message_types_by_name['WeightedCommonStatistics'] = _WEIGHTEDCOMMONSTATISTICS
DESCRIPTOR.message_types_by_name['CustomStatistic'] = _CUSTOMSTATISTIC
DESCRIPTOR.message_types_by_name['NumericStatistics'] = _NUMERICSTATISTICS
DESCRIPTOR.message_types_by_name['StringStatistics'] = _STRINGSTATISTICS
DESCRIPTOR.message_types_by_name['WeightedNumericStatistics'] = _WEIGHTEDNUMERICSTATISTICS
DESCRIPTOR.message_types_by_name['WeightedStringStatistics'] = _WEIGHTEDSTRINGSTATISTICS
DESCRIPTOR.message_types_by_name['BytesStatistics'] = _BYTESSTATISTICS
DESCRIPTOR.message_types_by_name['StructStatistics'] = _STRUCTSTATISTICS
DESCRIPTOR.message_types_by_name['CommonStatistics'] = _COMMONSTATISTICS
DESCRIPTOR.message_types_by_name['Histogram'] = _HISTOGRAM
DESCRIPTOR.message_types_by_name['RankHistogram'] = _RANKHISTOGRAM
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DatasetFeatureStatisticsList = _reflection.GeneratedProtocolMessageType('DatasetFeatureStatisticsList', (_message.Message,), {
  'DESCRIPTOR' : _DATASETFEATURESTATISTICSLIST,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.DatasetFeatureStatisticsList)
  })
_sym_db.RegisterMessage(DatasetFeatureStatisticsList)

DatasetFeatureStatistics = _reflection.GeneratedProtocolMessageType('DatasetFeatureStatistics', (_message.Message,), {
  'DESCRIPTOR' : _DATASETFEATURESTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.DatasetFeatureStatistics)
  })
_sym_db.RegisterMessage(DatasetFeatureStatistics)

CrossFeatureStatistics = _reflection.GeneratedProtocolMessageType('CrossFeatureStatistics', (_message.Message,), {
  'DESCRIPTOR' : _CROSSFEATURESTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.CrossFeatureStatistics)
  })
_sym_db.RegisterMessage(CrossFeatureStatistics)

NumericCrossStatistics = _reflection.GeneratedProtocolMessageType('NumericCrossStatistics', (_message.Message,), {
  'DESCRIPTOR' : _NUMERICCROSSSTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.NumericCrossStatistics)
  })
_sym_db.RegisterMessage(NumericCrossStatistics)

CategoricalCrossStatistics = _reflection.GeneratedProtocolMessageType('CategoricalCrossStatistics', (_message.Message,), {
  'DESCRIPTOR' : _CATEGORICALCROSSSTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.CategoricalCrossStatistics)
  })
_sym_db.RegisterMessage(CategoricalCrossStatistics)

LiftStatistics = _reflection.GeneratedProtocolMessageType('LiftStatistics', (_message.Message,), {
  'DESCRIPTOR' : _LIFTSTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.LiftStatistics)
  })
_sym_db.RegisterMessage(LiftStatistics)

LiftSeries = _reflection.GeneratedProtocolMessageType('LiftSeries', (_message.Message,), {

  'Bucket' : _reflection.GeneratedProtocolMessageType('Bucket', (_message.Message,), {
    'DESCRIPTOR' : _LIFTSERIES_BUCKET,
    '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
    # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.LiftSeries.Bucket)
    })
  ,

  'LiftValue' : _reflection.GeneratedProtocolMessageType('LiftValue', (_message.Message,), {
    'DESCRIPTOR' : _LIFTSERIES_LIFTVALUE,
    '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
    # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.LiftSeries.LiftValue)
    })
  ,
  'DESCRIPTOR' : _LIFTSERIES,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.LiftSeries)
  })
_sym_db.RegisterMessage(LiftSeries)
_sym_db.RegisterMessage(LiftSeries.Bucket)
_sym_db.RegisterMessage(LiftSeries.LiftValue)

FeatureNameStatistics = _reflection.GeneratedProtocolMessageType('FeatureNameStatistics', (_message.Message,), {
  'DESCRIPTOR' : _FEATURENAMESTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.FeatureNameStatistics)
  })
_sym_db.RegisterMessage(FeatureNameStatistics)

WeightedCommonStatistics = _reflection.GeneratedProtocolMessageType('WeightedCommonStatistics', (_message.Message,), {
  'DESCRIPTOR' : _WEIGHTEDCOMMONSTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.WeightedCommonStatistics)
  })
_sym_db.RegisterMessage(WeightedCommonStatistics)

CustomStatistic = _reflection.GeneratedProtocolMessageType('CustomStatistic', (_message.Message,), {
  'DESCRIPTOR' : _CUSTOMSTATISTIC,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.CustomStatistic)
  })
_sym_db.RegisterMessage(CustomStatistic)

NumericStatistics = _reflection.GeneratedProtocolMessageType('NumericStatistics', (_message.Message,), {
  'DESCRIPTOR' : _NUMERICSTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.NumericStatistics)
  })
_sym_db.RegisterMessage(NumericStatistics)

StringStatistics = _reflection.GeneratedProtocolMessageType('StringStatistics', (_message.Message,), {

  'FreqAndValue' : _reflection.GeneratedProtocolMessageType('FreqAndValue', (_message.Message,), {
    'DESCRIPTOR' : _STRINGSTATISTICS_FREQANDVALUE,
    '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
    # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.StringStatistics.FreqAndValue)
    })
  ,
  'DESCRIPTOR' : _STRINGSTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.StringStatistics)
  })
_sym_db.RegisterMessage(StringStatistics)
_sym_db.RegisterMessage(StringStatistics.FreqAndValue)

WeightedNumericStatistics = _reflection.GeneratedProtocolMessageType('WeightedNumericStatistics', (_message.Message,), {
  'DESCRIPTOR' : _WEIGHTEDNUMERICSTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.WeightedNumericStatistics)
  })
_sym_db.RegisterMessage(WeightedNumericStatistics)

WeightedStringStatistics = _reflection.GeneratedProtocolMessageType('WeightedStringStatistics', (_message.Message,), {
  'DESCRIPTOR' : _WEIGHTEDSTRINGSTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.WeightedStringStatistics)
  })
_sym_db.RegisterMessage(WeightedStringStatistics)

BytesStatistics = _reflection.GeneratedProtocolMessageType('BytesStatistics', (_message.Message,), {
  'DESCRIPTOR' : _BYTESSTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.BytesStatistics)
  })
_sym_db.RegisterMessage(BytesStatistics)

StructStatistics = _reflection.GeneratedProtocolMessageType('StructStatistics', (_message.Message,), {
  'DESCRIPTOR' : _STRUCTSTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.StructStatistics)
  })
_sym_db.RegisterMessage(StructStatistics)

CommonStatistics = _reflection.GeneratedProtocolMessageType('CommonStatistics', (_message.Message,), {
  'DESCRIPTOR' : _COMMONSTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.CommonStatistics)
  })
_sym_db.RegisterMessage(CommonStatistics)

Histogram = _reflection.GeneratedProtocolMessageType('Histogram', (_message.Message,), {

  'Bucket' : _reflection.GeneratedProtocolMessageType('Bucket', (_message.Message,), {
    'DESCRIPTOR' : _HISTOGRAM_BUCKET,
    '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
    # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.Histogram.Bucket)
    })
  ,
  'DESCRIPTOR' : _HISTOGRAM,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.Histogram)
  })
_sym_db.RegisterMessage(Histogram)
_sym_db.RegisterMessage(Histogram.Bucket)

RankHistogram = _reflection.GeneratedProtocolMessageType('RankHistogram', (_message.Message,), {

  'Bucket' : _reflection.GeneratedProtocolMessageType('Bucket', (_message.Message,), {
    'DESCRIPTOR' : _RANKHISTOGRAM_BUCKET,
    '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
    # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.RankHistogram.Bucket)
    })
  ,
  'DESCRIPTOR' : _RANKHISTOGRAM,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.RankHistogram)
  })
_sym_db.RegisterMessage(RankHistogram)
_sym_db.RegisterMessage(RankHistogram.Bucket)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
