# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/OnDemandFeatureView.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from feast.protos.feast.core import FeatureView_pb2 as feast_dot_core_dot_FeatureView__pb2
from feast.protos.feast.core import Feature_pb2 as feast_dot_core_dot_Feature__pb2
from feast.protos.feast.core import DataSource_pb2 as feast_dot_core_dot_DataSource__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='feast/core/OnDemandFeatureView.proto',
  package='feast.core',
  syntax='proto3',
  serialized_options=b'\n\020feast.proto.coreB\030OnDemandFeatureViewProtoZ3github.com/feast-dev/feast/sdk/go/protos/feast/core',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n$feast/core/OnDemandFeatureView.proto\x12\nfeast.core\x1a\x1c\x66\x65\x61st/core/FeatureView.proto\x1a\x18\x66\x65\x61st/core/Feature.proto\x1a\x1b\x66\x65\x61st/core/DataSource.proto\"H\n\x13OnDemandFeatureView\x12\x31\n\x04spec\x18\x01 \x01(\x0b\x32#.feast.core.OnDemandFeatureViewSpec\"\xb0\x02\n\x17OnDemandFeatureViewSpec\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12+\n\x08\x66\x65\x61tures\x18\x03 \x03(\x0b\x32\x19.feast.core.FeatureSpecV2\x12?\n\x06inputs\x18\x04 \x03(\x0b\x32/.feast.core.OnDemandFeatureViewSpec.InputsEntry\x12>\n\x15user_defined_function\x18\x05 \x01(\x0b\x32\x1f.feast.core.UserDefinedFunction\x1aH\n\x0bInputsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.feast.core.OnDemandInput:\x02\x38\x01\"\x80\x01\n\rOnDemandInput\x12/\n\x0c\x66\x65\x61ture_view\x18\x01 \x01(\x0b\x32\x17.feast.core.FeatureViewH\x00\x12\x35\n\x13request_data_source\x18\x02 \x01(\x0b\x32\x16.feast.core.DataSourceH\x00\x42\x07\n\x05input\"1\n\x13UserDefinedFunction\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04\x62ody\x18\x02 \x01(\x0c\x42\x61\n\x10\x66\x65\x61st.proto.coreB\x18OnDemandFeatureViewProtoZ3github.com/feast-dev/feast/sdk/go/protos/feast/coreb\x06proto3'
  ,
  dependencies=[feast_dot_core_dot_FeatureView__pb2.DESCRIPTOR,feast_dot_core_dot_Feature__pb2.DESCRIPTOR,feast_dot_core_dot_DataSource__pb2.DESCRIPTOR,])




_ONDEMANDFEATUREVIEW = _descriptor.Descriptor(
  name='OnDemandFeatureView',
  full_name='feast.core.OnDemandFeatureView',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='spec', full_name='feast.core.OnDemandFeatureView.spec', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=137,
  serialized_end=209,
)


_ONDEMANDFEATUREVIEWSPEC_INPUTSENTRY = _descriptor.Descriptor(
  name='InputsEntry',
  full_name='feast.core.OnDemandFeatureViewSpec.InputsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='feast.core.OnDemandFeatureViewSpec.InputsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='feast.core.OnDemandFeatureViewSpec.InputsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=444,
  serialized_end=516,
)

_ONDEMANDFEATUREVIEWSPEC = _descriptor.Descriptor(
  name='OnDemandFeatureViewSpec',
  full_name='feast.core.OnDemandFeatureViewSpec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='feast.core.OnDemandFeatureViewSpec.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='project', full_name='feast.core.OnDemandFeatureViewSpec.project', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='features', full_name='feast.core.OnDemandFeatureViewSpec.features', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='inputs', full_name='feast.core.OnDemandFeatureViewSpec.inputs', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='user_defined_function', full_name='feast.core.OnDemandFeatureViewSpec.user_defined_function', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_ONDEMANDFEATUREVIEWSPEC_INPUTSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=212,
  serialized_end=516,
)


_ONDEMANDINPUT = _descriptor.Descriptor(
  name='OnDemandInput',
  full_name='feast.core.OnDemandInput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='feature_view', full_name='feast.core.OnDemandInput.feature_view', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='request_data_source', full_name='feast.core.OnDemandInput.request_data_source', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='input', full_name='feast.core.OnDemandInput.input',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=519,
  serialized_end=647,
)


_USERDEFINEDFUNCTION = _descriptor.Descriptor(
  name='UserDefinedFunction',
  full_name='feast.core.UserDefinedFunction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='feast.core.UserDefinedFunction.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='body', full_name='feast.core.UserDefinedFunction.body', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=649,
  serialized_end=698,
)

_ONDEMANDFEATUREVIEW.fields_by_name['spec'].message_type = _ONDEMANDFEATUREVIEWSPEC
_ONDEMANDFEATUREVIEWSPEC_INPUTSENTRY.fields_by_name['value'].message_type = _ONDEMANDINPUT
_ONDEMANDFEATUREVIEWSPEC_INPUTSENTRY.containing_type = _ONDEMANDFEATUREVIEWSPEC
_ONDEMANDFEATUREVIEWSPEC.fields_by_name['features'].message_type = feast_dot_core_dot_Feature__pb2._FEATURESPECV2
_ONDEMANDFEATUREVIEWSPEC.fields_by_name['inputs'].message_type = _ONDEMANDFEATUREVIEWSPEC_INPUTSENTRY
_ONDEMANDFEATUREVIEWSPEC.fields_by_name['user_defined_function'].message_type = _USERDEFINEDFUNCTION
_ONDEMANDINPUT.fields_by_name['feature_view'].message_type = feast_dot_core_dot_FeatureView__pb2._FEATUREVIEW
_ONDEMANDINPUT.fields_by_name['request_data_source'].message_type = feast_dot_core_dot_DataSource__pb2._DATASOURCE
_ONDEMANDINPUT.oneofs_by_name['input'].fields.append(
  _ONDEMANDINPUT.fields_by_name['feature_view'])
_ONDEMANDINPUT.fields_by_name['feature_view'].containing_oneof = _ONDEMANDINPUT.oneofs_by_name['input']
_ONDEMANDINPUT.oneofs_by_name['input'].fields.append(
  _ONDEMANDINPUT.fields_by_name['request_data_source'])
_ONDEMANDINPUT.fields_by_name['request_data_source'].containing_oneof = _ONDEMANDINPUT.oneofs_by_name['input']
DESCRIPTOR.message_types_by_name['OnDemandFeatureView'] = _ONDEMANDFEATUREVIEW
DESCRIPTOR.message_types_by_name['OnDemandFeatureViewSpec'] = _ONDEMANDFEATUREVIEWSPEC
DESCRIPTOR.message_types_by_name['OnDemandInput'] = _ONDEMANDINPUT
DESCRIPTOR.message_types_by_name['UserDefinedFunction'] = _USERDEFINEDFUNCTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

OnDemandFeatureView = _reflection.GeneratedProtocolMessageType('OnDemandFeatureView', (_message.Message,), {
  'DESCRIPTOR' : _ONDEMANDFEATUREVIEW,
  '__module__' : 'feast.core.OnDemandFeatureView_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.OnDemandFeatureView)
  })
_sym_db.RegisterMessage(OnDemandFeatureView)

OnDemandFeatureViewSpec = _reflection.GeneratedProtocolMessageType('OnDemandFeatureViewSpec', (_message.Message,), {

  'InputsEntry' : _reflection.GeneratedProtocolMessageType('InputsEntry', (_message.Message,), {
    'DESCRIPTOR' : _ONDEMANDFEATUREVIEWSPEC_INPUTSENTRY,
    '__module__' : 'feast.core.OnDemandFeatureView_pb2'
    # @@protoc_insertion_point(class_scope:feast.core.OnDemandFeatureViewSpec.InputsEntry)
    })
  ,
  'DESCRIPTOR' : _ONDEMANDFEATUREVIEWSPEC,
  '__module__' : 'feast.core.OnDemandFeatureView_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.OnDemandFeatureViewSpec)
  })
_sym_db.RegisterMessage(OnDemandFeatureViewSpec)
_sym_db.RegisterMessage(OnDemandFeatureViewSpec.InputsEntry)

OnDemandInput = _reflection.GeneratedProtocolMessageType('OnDemandInput', (_message.Message,), {
  'DESCRIPTOR' : _ONDEMANDINPUT,
  '__module__' : 'feast.core.OnDemandFeatureView_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.OnDemandInput)
  })
_sym_db.RegisterMessage(OnDemandInput)

UserDefinedFunction = _reflection.GeneratedProtocolMessageType('UserDefinedFunction', (_message.Message,), {
  'DESCRIPTOR' : _USERDEFINEDFUNCTION,
  '__module__' : 'feast.core.OnDemandFeatureView_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.UserDefinedFunction)
  })
_sym_db.RegisterMessage(UserDefinedFunction)


DESCRIPTOR._options = None
_ONDEMANDFEATUREVIEWSPEC_INPUTSENTRY._options = None
# @@protoc_insertion_point(module_scope)
